# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..specialized import BRAINSABC


def test_BRAINSABC_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        atlasDefinition=dict(
            argstr='--atlasDefinition %s',
            extensions=None,
        ),
        atlasToSubjectInitialTransform=dict(
            argstr='--atlasToSubjectInitialTransform %s',
            hash_files=False,
        ),
        atlasToSubjectTransform=dict(
            argstr='--atlasToSubjectTransform %s',
            hash_files=False,
        ),
        atlasToSubjectTransformType=dict(
            argstr='--atlasToSubjectTransformType %s', ),
        atlasWarpingOff=dict(argstr='--atlasWarpingOff ', ),
        debuglevel=dict(argstr='--debuglevel %d', ),
        defaultSuffix=dict(argstr='--defaultSuffix %s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filterIteration=dict(argstr='--filterIteration %d', ),
        filterMethod=dict(argstr='--filterMethod %s', ),
        filterTimeStep=dict(argstr='--filterTimeStep %f', ),
        gridSize=dict(
            argstr='--gridSize %s',
            sep=',',
        ),
        implicitOutputs=dict(
            argstr='--implicitOutputs %s...',
            hash_files=False,
        ),
        inputVolumeTypes=dict(
            argstr='--inputVolumeTypes %s',
            sep=',',
        ),
        inputVolumes=dict(argstr='--inputVolumes %s...', ),
        interpolationMode=dict(argstr='--interpolationMode %s', ),
        maxBiasDegree=dict(argstr='--maxBiasDegree %d', ),
        maxIterations=dict(argstr='--maxIterations %d', ),
        medianFilterSize=dict(
            argstr='--medianFilterSize %s',
            sep=',',
        ),
        numberOfSubSamplesInEachPlugArea=dict(
            argstr='--numberOfSubSamplesInEachPlugArea %s',
            sep=',',
        ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputDir=dict(
            argstr='--outputDir %s',
            hash_files=False,
        ),
        outputDirtyLabels=dict(
            argstr='--outputDirtyLabels %s',
            hash_files=False,
        ),
        outputFormat=dict(argstr='--outputFormat %s', ),
        outputLabels=dict(
            argstr='--outputLabels %s',
            hash_files=False,
        ),
        outputVolumes=dict(
            argstr='--outputVolumes %s...',
            hash_files=False,
        ),
        posteriorTemplate=dict(argstr='--posteriorTemplate %s', ),
        purePlugsThreshold=dict(argstr='--purePlugsThreshold %f', ),
        restoreState=dict(
            argstr='--restoreState %s',
            extensions=None,
        ),
        saveState=dict(
            argstr='--saveState %s',
            hash_files=False,
        ),
        subjectIntermodeTransformType=dict(
            argstr='--subjectIntermodeTransformType %s', ),
        useKNN=dict(argstr='--useKNN ', ),
        writeLess=dict(argstr='--writeLess ', ),
    )
    inputs = BRAINSABC.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSABC_outputs():
    output_map = dict(
        atlasToSubjectInitialTransform=dict(extensions=None, ),
        atlasToSubjectTransform=dict(extensions=None, ),
        implicitOutputs=dict(),
        outputDir=dict(),
        outputDirtyLabels=dict(extensions=None, ),
        outputLabels=dict(extensions=None, ),
        outputVolumes=dict(),
        saveState=dict(extensions=None, ),
    )
    outputs = BRAINSABC.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ComputeTDI


def test_ComputeTDI_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        contrast=dict(argstr='-constrast %s', ),
        data_type=dict(argstr='-datatype %s', ),
        dixel=dict(
            argstr='-dixel %s',
            extensions=None,
        ),
        ends_only=dict(argstr='-ends_only', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fwhm_tck=dict(argstr='-fwhm_tck %f', ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        in_map=dict(
            argstr='-image %s',
            extensions=None,
        ),
        map_zero=dict(argstr='-map_zero', ),
        max_tod=dict(argstr='-tod %d', ),
        nthreads=dict(
            argstr='-nthreads %d',
            nohash=True,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            position=-1,
            usedefault=True,
        ),
        precise=dict(argstr='-precise', ),
        reference=dict(
            argstr='-template %s',
            extensions=None,
        ),
        stat_tck=dict(argstr='-stat_tck %s', ),
        stat_vox=dict(argstr='-stat_vox %s', ),
        tck_weights=dict(
            argstr='-tck_weights_in %s',
            extensions=None,
        ),
        upsample=dict(argstr='-upsample %d', ),
        use_dec=dict(argstr='-dec', ),
        vox_size=dict(
            argstr='-vox %s',
            sep=',',
        ),
    )
    inputs = ComputeTDI.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ComputeTDI_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = ComputeTDI.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..connectivity import BuildConnectome


def test_BuildConnectome_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        in_parc=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        in_scalar=dict(
            argstr='-image %s',
            extensions=None,
        ),
        in_weights=dict(
            argstr='-tck_weights_in %s',
            extensions=None,
        ),
        keep_unassigned=dict(argstr='-keep_unassigned', ),
        metric=dict(argstr='-metric %s', ),
        nthreads=dict(
            argstr='-nthreads %d',
            nohash=True,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-1,
            usedefault=True,
        ),
        search_forward=dict(argstr='-assignment_forward_search %f', ),
        search_radius=dict(argstr='-assignment_radial_search %f', ),
        search_reverse=dict(argstr='-assignment_reverse_search %f', ),
        vox_lookup=dict(argstr='-assignment_voxel_lookup', ),
        zero_diagonal=dict(argstr='-zero_diagonal', ),
    )
    inputs = BuildConnectome.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BuildConnectome_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = BuildConnectome.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Utilities to keep track of performance
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os
import threading
from time import time
try:
    import psutil
except ImportError as exc:
    psutil = None

from builtins import open, range
from .. import config, logging

proflogger = logging.getLogger('nipype.utils')
resource_monitor = config.resource_monitor

# Init variables
_MB = 1024.0**2


class ResourceMonitor(threading.Thread):
    """
    A ``Thread`` to monitor a specific PID with a certain frequence
    to a file
    """

    def __init__(self, pid, freq=5, fname=None, python=True):
        # Make sure psutil is imported
        import psutil

        if freq < 0.2:
            raise RuntimeError(
                'Frequency (%0.2fs) cannot be lower than 0.2s' % freq)

        if fname is None:
            fname = '.proc-%d_time-%s_freq-%0.2f' % (pid, time(), freq)
        self._fname = os.path.abspath(fname)
        self._logfile = open(self._fname, 'w')
        self._freq = freq
        self._python = python

        # Leave process initialized and make first sample
        self._process = psutil.Process(pid)
        self._sample(cpu_interval=0.2)

        # Start thread
        threading.Thread.__init__(self)
        self._event = threading.Event()

    @property
    def fname(self):
        """Get/set the internal filename"""
        return self._fname

    def stop(self):
        """Stop monitoring"""
        if not self._event.is_set():
            self._event.set()
            self.join()
            self._sample()
            self._logfile.flush()
            self._logfile.close()

    def _sample(self, cpu_interval=None):
        cpu = 0.0
        rss = 0.0
        vms = 0.0
        try:
            with self._process.oneshot():
                cpu += self._process.cpu_percent(interval=cpu_interval)
                mem_info = self._process.memory_info()
                rss += mem_info.rss
                vms += mem_info.vms
        except psutil.NoSuchProcess:
            pass

        # Iterate through child processes and get number of their threads
        try:
            children = self._process.children(recursive=True)
        except psutil.NoSuchProcess:
            children = []

        for child in children:
            try:
                with child.oneshot():
                    cpu += child.cpu_percent()
                    mem_info = child.memory_info()
                    rss += mem_info.rss
                    vms += mem_info.vms
            except psutil.NoSuchProcess:
                pass

        print(
            '%f,%f,%f,%f' % (time(), cpu, rss / _MB, vms / _MB),
            file=self._logfile)
        self._logfile.flush()

    def run(self):
        """Core monitoring function, called by start()"""
        start_time = time()
        wait_til = start_time
        while not self._event.is_set():
            self._sample()
            wait_til += self._freq
            self._event.wait(max(0, wait_til - time()))


# Log node stats function
def log_nodes_cb(node, status):
    """Function to record node run statistics to a log file as json
    dictionaries

    Parameters
    ----------
    node : nipype.pipeline.engine.Node
        the node being logged
    status : string
        acceptable values are 'start', 'end'; otherwise it is
        considered and error

    Returns
    -------
    None
        this function does not return any values, it logs the node
        status info to the callback logger
    """

    if status != 'end':
        return

    # Import packages
    import logging
    import json

    status_dict = {
        'name': node.name,
        'id': node._id,
        'start': getattr(node.result.runtime, 'startTime'),
        'finish': getattr(node.result.runtime, 'endTime'),
        'duration': getattr(node.result.runtime, 'duration'),
        'runtime_threads': getattr(node.result.runtime, 'cpu_percent', 'N/A'),
        'runtime_memory_gb': getattr(node.result.runtime, 'mem_peak_gb',
                                     'N/A'),
        'estimated_memory_gb': node.mem_gb,
        'num_threads': node.n_procs,
    }

    if status_dict['start'] is None or status_dict['finish'] is None:
        status_dict['error'] = True

    # Dump string to log
    logging.getLogger('callback').debug(json.dumps(status_dict))


# Get total system RAM
def get_system_total_memory_gb():
    """
    Function to get the total RAM of the running system in GB
    """

    # Import packages
    import os
    import sys

    # Get memory
    if 'linux' in sys.platform:
        with open('/proc/meminfo', 'r') as f_in:
            meminfo_lines = f_in.readlines()
            mem_total_line = [
                line for line in meminfo_lines if 'MemTotal' in line
            ][0]
            mem_total = float(mem_total_line.split()[1])
            memory_gb = mem_total / (1024.0**2)
    elif 'darwin' in sys.platform:
        mem_str = os.popen('sysctl hw.memsize').read().strip().split(' ')[-1]
        memory_gb = float(mem_str) / (1024.0**3)
    else:
        err_msg = 'System platform: %s is not supported'
        raise Exception(err_msg)

    # Return memory
    return memory_gb


# Get max resources used for process
def get_max_resources_used(pid, mem_mb, num_threads, pyfunc=False):
    """
    Function to get the RAM and threads utilized by a given process

    Parameters
    ----------
    pid : integer
        the process ID of process to profile
    mem_mb : float
        the high memory watermark so far during process execution (in MB)
    num_threads: int
        the high thread watermark so far during process execution

    Returns
    -------
    mem_mb : float
        the new high memory watermark of process (MB)
    num_threads : float
        the new high thread watermark of process
    """

    if not resource_monitor:
        raise RuntimeError('Attempted to measure resources with option '
                           '"monitoring.enabled" set off.')

    try:
        mem_mb = max(mem_mb, _get_ram_mb(pid, pyfunc=pyfunc))
        num_threads = max(num_threads, _get_num_threads(pid))
    except Exception as exc:
        proflogger.info('Could not get resources used by process.\n%s', exc)

    return mem_mb, num_threads


# Get number of threads for process
def _get_num_threads(pid):
    """
    Function to get the number of threads a process is using

    Parameters
    ----------
    pid : integer
        the process ID of process to profile

    Returns
    -------
    num_threads : int
        the number of threads that the process is using

    """

    try:
        proc = psutil.Process(pid)
        # If process is running
        if proc.status() == psutil.STATUS_RUNNING:
            num_threads = proc.num_threads()
        elif proc.num_threads() > 1:
            tprocs = [psutil.Process(thr.id) for thr in proc.threads()]
            alive_tprocs = [
                tproc for tproc in tprocs
                if tproc.status() == psutil.STATUS_RUNNING
            ]
            num_threads = len(alive_tprocs)
        else:
            num_threads = 1

        child_threads = 0
        # Iterate through child processes and get number of their threads
        for child in proc.children(recursive=True):
            # Leaf process
            if len(child.children()) == 0:
                # If process is running, get its number of threads
                if child.status() == psutil.STATUS_RUNNING:
                    child_thr = child.num_threads()
                # If its not necessarily running, but still multi-threaded
                elif child.num_threads() > 1:
                    # Cast each thread as a process and check for only running
                    tprocs = [
                        psutil.Process(thr.id) for thr in child.threads()
                    ]
                    alive_tprocs = [
                        tproc for tproc in tprocs
                        if tproc.status() == psutil.STATUS_RUNNING
                    ]
                    child_thr = len(alive_tprocs)
                # Otherwise, no threads are running
                else:
                    child_thr = 0
                # Increment child threads
                child_threads += child_thr
    except psutil.NoSuchProcess:
        return None

    # Number of threads is max between found active children and parent
    num_threads = max(child_threads, num_threads)

    # Return number of threads found
    return num_threads


# Get ram usage of process
def _get_ram_mb(pid, pyfunc=False):
    """
    Function to get the RAM usage of a process and its children
    Reference: http://ftp.dev411.com/t/python/python-list/095thexx8g/\
multiprocessing-forking-memory-usage

    Parameters
    ----------
    pid : integer
        the PID of the process to get RAM usage of
    pyfunc : boolean (optional); default=False
        a flag to indicate if the process is a python function;
        when Pythons are multithreaded via multiprocess or threading,
        children functions include their own memory + parents. if this
        is set, the parent memory will removed from children memories


    Returns
    -------
    mem_mb : float
        the memory RAM in MB utilized by the process PID

    """
    try:
        # Init parent
        parent = psutil.Process(pid)
        # Get memory of parent
        parent_mem = parent.memory_info().rss
        mem_mb = parent_mem / _MB
        # Iterate through child processes
        for child in parent.children(recursive=True):
            child_mem = child.memory_info().rss
            if pyfunc:
                child_mem -= parent_mem
            mem_mb += child_mem / _MB
    except psutil.NoSuchProcess:
        return None

    # Return memory
    return mem_mb


def _use_cpu(x):
    ctr = 0
    while ctr < 1e7:
        ctr += 1
        x * x


# Spin multiple threads
def _use_resources(n_procs, mem_gb):
    """
    Function to execute multiple use_gb_ram functions in parallel
    """
    import os
    import sys
    import psutil
    from multiprocessing import Pool
    from nipype import logging
    from nipype.utils.profiler import _use_cpu

    iflogger = logging.getLogger('nipype.interface')

    # Getsize of one character string
    BSIZE = sys.getsizeof('  ') - sys.getsizeof(' ')
    BOFFSET = sys.getsizeof('')
    _GB = 1024.0**3

    def _use_gb_ram(mem_gb):
        """A test function to consume mem_gb GB of RAM"""
        num_bytes = int(mem_gb * _GB)
        # Eat mem_gb GB of memory for 1 second
        gb_str = ' ' * ((num_bytes - BOFFSET) // BSIZE)
        assert sys.getsizeof(gb_str) == num_bytes
        return gb_str

    # Measure the amount of memory this process already holds
    p = psutil.Process(os.getpid())
    mem_offset = p.memory_info().rss / _GB
    big_str = _use_gb_ram(mem_gb - mem_offset)
    _use_cpu(5)
    mem_total = p.memory_info().rss / _GB
    del big_str
    iflogger.info('[%d] Memory offset %0.2fGB, total %0.2fGB', os.getpid(),
                  mem_offset, mem_total)

    if n_procs > 1:
        pool = Pool(n_procs)
        pool.map(_use_cpu, range(n_procs))
    return True



# -*- coding: utf-8 -*-
"""Parallel workflow execution via PBS/Torque
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import str, open

import os
from time import sleep

from ... import logging
from ...interfaces.base import CommandLine
from .base import SGELikeBatchManagerBase, logger

iflogger = logging.getLogger('nipype.interface')


class PBSPlugin(SGELikeBatchManagerBase):
    """Execute using PBS/Torque

    The plugin_args input to run can be used to control the SGE execution.
    Currently supported options are:

    - template : template to use for batch job submission
    - qsub_args : arguments to be prepended to the job execution script in the
                  qsub call
    - max_jobname_len: maximum length of the job name.  Default 15.

    """

    # Addtional class variables
    _max_jobname_len = 15

    def __init__(self, **kwargs):
        template = """
#PBS -V
        """
        self._retry_timeout = 2
        self._max_tries = 2
        self._max_jobname_length = 15
        if 'plugin_args' in kwargs and kwargs['plugin_args']:
            if 'retry_timeout' in kwargs['plugin_args']:
                self._retry_timeout = kwargs['plugin_args']['retry_timeout']
            if 'max_tries' in kwargs['plugin_args']:
                self._max_tries = kwargs['plugin_args']['max_tries']
            if 'max_jobname_len' in kwargs['plugin_args']:
                self._max_jobname_len = kwargs['plugin_args'][
                    'max_jobname_len']
        super(PBSPlugin, self).__init__(template, **kwargs)

    def _is_pending(self, taskid):
        result = CommandLine('qstat -f {}'.format(taskid),
                             environ=dict(os.environ),
                             terminal_output='file_split',
                             resource_monitor=False,
                             ignore_exception=True).run()

        stdout = result.runtime.stdout
        stderr = result.runtime.stderr
        errmsg = 'Unknown Job Id'
        success = 'Job has finished'
        if (success in stderr) or ('job_state = C' in stdout):
            return False
        else:
            return errmsg not in stderr

    def _submit_batchtask(self, scriptfile, node):
        cmd = CommandLine(
            'qsub',
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output='allatonce')
        path = os.path.dirname(scriptfile)
        qsubargs = ''
        if self._qsub_args:
            qsubargs = self._qsub_args
        if 'qsub_args' in node.plugin_args:
            if 'overwrite' in node.plugin_args and \
               node.plugin_args['overwrite']:
                qsubargs = node.plugin_args['qsub_args']
            else:
                qsubargs += (" " + node.plugin_args['qsub_args'])
        if '-o' not in qsubargs:
            qsubargs = '%s -o %s' % (qsubargs, path)
        if '-e' not in qsubargs:
            qsubargs = '%s -e %s' % (qsubargs, path)
        if node._hierarchy:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._hierarchy,
                                node._id))
        else:
            jobname = '.'.join((dict(os.environ)['LOGNAME'], node._id))
        jobnameitems = jobname.split('.')
        jobnameitems.reverse()
        jobname = '.'.join(jobnameitems)
        jobname = jobname[0:self._max_jobname_len]
        cmd.inputs.args = '%s -N %s %s' % (qsubargs, jobname, scriptfile)

        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName('CRITICAL'))
        tries = 0
        while True:
            try:
                result = cmd.run()
            except Exception as e:
                if tries < self._max_tries:
                    tries += 1
                    # sleep 2 seconds and try again.
                    sleep(self._retry_timeout)
                else:
                    iflogger.setLevel(oldlevel)
                    raise RuntimeError(
                        'Could not submit pbs task for node {}\n{}'.format(
                            node._id, e))
            else:
                break
        iflogger.setLevel(oldlevel)
        # retrieve pbs taskid
        taskid = result.runtime.stdout.split('.')[0]
        self._pending[taskid] = node.output_dir()
        logger.debug('submitted pbs task: {} for node {}'.format(
            taskid, node._id))

        return taskid



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The minc module provides classes for interfacing with the `MINC
<http://www.bic.mni.mcgill.ca/ServicesSoftware/MINC>`_ command line tools. This
module was written to work with MINC version 2.2.00.

Author: Carlo Hamalainen <carlo@carlo-hamalainen.net>
        http://carlo-hamalainen.net
"""

from .base import (Info)

from .minc import (
    Average,
    BBox,
    Beast,
    BestLinReg,
    BigAverage,
    Blob,
    Blur,
    Calc,
    Convert,
    Copy,
    Dump,
    Extract,
    Gennlxfm,
    Math,
    NlpFit,
    Norm,
    Pik,
    Resample,
    Reshape,
    ToEcat,
    ToRaw,
    Volcentre,
    Voliso,
    Volpad,
    VolSymm,
    XfmAvg,
    XfmConcat,
    XfmInvert,
)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The freesurfer module provides basic functions for interfacing with
freesurfer tools.

Currently these tools are supported:

     * Dicom2Nifti: using mri_convert
     * Resample: using mri_convert

Examples
--------
See the docstrings for the individual classes for 'working' examples.

"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open, object, str

import os

from ... import LooseVersion
from ...utils.filemanip import fname_presuffix
from ..base import (CommandLine, Directory, CommandLineInputSpec, isdefined,
                    traits, TraitedSpec, File, PackageInfo)

__docformat__ = 'restructuredtext'


class Info(PackageInfo):
    """ Freesurfer subject directory and version information.

    Examples
    --------

    >>> from nipype.interfaces.freesurfer import Info
    >>> Info.version()  # doctest: +SKIP
    >>> Info.subjectsdir()  # doctest: +SKIP

    """
    if os.getenv('FREESURFER_HOME'):
        version_file = os.path.join(
            os.getenv('FREESURFER_HOME'), 'build-stamp.txt')

    @staticmethod
    def parse_version(raw_info):
        return raw_info.splitlines()[0]

    @classmethod
    def looseversion(cls):
        """ Return a comparable version object

        If no version found, use LooseVersion('0.0.0')
        """
        ver = cls.version()
        if ver is None:
            return LooseVersion('0.0.0')

        vinfo = ver.rstrip().split('-')
        try:
            int(vinfo[-1], 16)
        except ValueError:
            githash = ''
        else:
            githash = '.' + vinfo[-1]

        # As of FreeSurfer v6.0.0, the final component is a githash
        if githash:
            if vinfo[3] == 'dev':
                # This will need updating when v6.0.1 comes out
                vstr = '6.0.0-dev' + githash
            elif vinfo[5][0] == 'v':
                vstr = vinfo[5][1:]
            else:
                raise RuntimeError('Unknown version string: ' + ver)
        # Retain pre-6.0.0 heuristics
        elif 'dev' in ver:
            vstr = vinfo[-1] + '-dev'
        else:
            vstr = ver.rstrip().split('-v')[-1]

        return LooseVersion(vstr)

    @classmethod
    def subjectsdir(cls):
        """Check the global SUBJECTS_DIR

        Parameters
        ----------

        subjects_dir :  string
            The system defined subjects directory

        Returns
        -------

        subject_dir : string
            Represents the current environment setting of SUBJECTS_DIR

        """
        if cls.version():
            return os.environ['SUBJECTS_DIR']
        return None


class FSTraitedSpec(CommandLineInputSpec):
    subjects_dir = Directory(exists=True, desc='subjects directory')


class FSCommand(CommandLine):
    """General support for FreeSurfer commands.

       Every FS command accepts 'subjects_dir' input.
    """

    input_spec = FSTraitedSpec

    _subjects_dir = None

    def __init__(self, **inputs):
        super(FSCommand, self).__init__(**inputs)
        self.inputs.on_trait_change(self._subjects_dir_update, 'subjects_dir')
        if not self._subjects_dir:
            self._subjects_dir = Info.subjectsdir()
        if not isdefined(self.inputs.subjects_dir) and self._subjects_dir:
            self.inputs.subjects_dir = self._subjects_dir
        self._subjects_dir_update()

    def _subjects_dir_update(self):
        if self.inputs.subjects_dir:
            self.inputs.environ.update({
                'SUBJECTS_DIR': self.inputs.subjects_dir
            })

    @classmethod
    def set_default_subjects_dir(cls, subjects_dir):
        cls._subjects_dir = subjects_dir

    def run(self, **inputs):
        if 'subjects_dir' in inputs:
            self.inputs.subjects_dir = inputs['subjects_dir']
        self._subjects_dir_update()
        return super(FSCommand, self).run(**inputs)

    def _gen_fname(self,
                   basename,
                   fname=None,
                   cwd=None,
                   suffix='_fs',
                   use_ext=True):
        '''Define a generic mapping for a single outfile

        The filename is potentially autogenerated by suffixing inputs.infile

        Parameters
        ----------
        basename : string (required)
            filename to base the new filename on
        fname : string
            if not None, just use this fname
        cwd : string
            prefix paths with cwd, otherwise os.getcwd()
        suffix : string
            default suffix
        '''
        if basename == '':
            msg = 'Unable to generate filename for command %s. ' % self.cmd
            msg += 'basename is not set!'
            raise ValueError(msg)
        if cwd is None:
            cwd = os.getcwd()
        fname = fname_presuffix(
            basename, suffix=suffix, use_ext=use_ext, newpath=cwd)
        return fname

    @property
    def version(self):
        ver = Info.looseversion()
        if ver > LooseVersion("0.0.0"):
            return ver.vstring


class FSSurfaceCommand(FSCommand):
    """Support for FreeSurfer surface-related functions.
    For some functions, if the output file is not specified starting with 'lh.'
    or 'rh.', FreeSurfer prepends the prefix from the input file to the output
    filename. Output out_file must be adjusted to accommodate this. By
    including the full path in the filename, we can also avoid this behavior.
    """

    @staticmethod
    def _associated_file(in_file, out_name):
        """Based on MRIsBuildFileName in freesurfer/utils/mrisurf.c

        If no path information is provided for out_name, use path and
        hemisphere (if also unspecified) from in_file to determine the path
        of the associated file.
        Use in_file prefix to indicate hemisphere for out_name, rather than
        inspecting the surface data structure.
        """
        path, base = os.path.split(out_name)
        if path == '':
            path, in_file = os.path.split(in_file)
            hemis = ('lh.', 'rh.')
            if in_file[:3] in hemis and base[:3] not in hemis:
                base = in_file[:3] + base
        return os.path.join(path, base)


class FSScriptCommand(FSCommand):
    """ Support for Freesurfer script commands with log terminal_output
    """
    _terminal_output = 'file'
    _always_run = False

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['log_file'] = os.path.abspath('output.nipype')
        return outputs


class FSScriptOutputSpec(TraitedSpec):
    log_file = File(
        'output.nipype', usedefault=True, exists=True, desc="The output log")


class FSTraitedSpecOpenMP(FSTraitedSpec):
    num_threads = traits.Int(desc='allows for specifying more threads')


class FSCommandOpenMP(FSCommand):
    """Support for FS commands that utilize OpenMP

    Sets the environment variable 'OMP_NUM_THREADS' to the number
    of threads specified by the input num_threads.
    """

    input_spec = FSTraitedSpecOpenMP

    _num_threads = None

    def __init__(self, **inputs):
        super(FSCommandOpenMP, self).__init__(**inputs)
        self.inputs.on_trait_change(self._num_threads_update, 'num_threads')
        if not self._num_threads:
            self._num_threads = os.environ.get('OMP_NUM_THREADS', None)
            if not self._num_threads:
                self._num_threads = os.environ.get('NSLOTS', None)
        if not isdefined(self.inputs.num_threads) and self._num_threads:
            self.inputs.num_threads = int(self._num_threads)
        self._num_threads_update()

    def _num_threads_update(self):
        if self.inputs.num_threads:
            self.inputs.environ.update({
                'OMP_NUM_THREADS':
                str(self.inputs.num_threads)
            })

    def run(self, **inputs):
        if 'num_threads' in inputs:
            self.inputs.num_threads = inputs['num_threads']
        self._num_threads_update()
        return super(FSCommandOpenMP, self).run(**inputs)


def no_freesurfer():
    """Checks if FreeSurfer is NOT installed
    used with skipif to skip tests that will
    fail if FreeSurfer is not installed"""

    if Info.version() is None:
        return True
    else:
        return False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import SVReg


def test_SVReg_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        atlasFilePrefix=dict(
            argstr="'%s'",
            position=1,
        ),
        curveMatchingInstructions=dict(argstr="'-cur %s'", ),
        dataSinkDelay=dict(argstr='%s', ),
        displayModuleName=dict(argstr="'-m'", ),
        displayTimestamps=dict(argstr="'-t'", ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        iterations=dict(argstr="'-H %d'", ),
        keepIntermediates=dict(argstr="'-k'", ),
        pialSurfaceMaskDilation=dict(argstr="'-D %d'", ),
        refineOutputs=dict(argstr="'-r'", ),
        shortMessages=dict(argstr="'-gui'", ),
        skipToIntensityReg=dict(argstr="'-p'", ),
        skipToVolumeReg=dict(argstr="'-s'", ),
        skipVolumetricProcessing=dict(argstr="'-S'", ),
        subjectFilePrefix=dict(
            argstr="'%s'",
            mandatory=True,
            position=0,
        ),
        useCerebrumMask=dict(argstr="'-C'", ),
        useManualMaskFile=dict(argstr="'-cbm'", ),
        useMultiThreading=dict(argstr="'-P'", ),
        useSingleThreading=dict(argstr="'-U'", ),
        verbosity0=dict(
            argstr="'-v0'",
            xor=('verbosity0', 'verbosity1', 'verbosity2'),
        ),
        verbosity1=dict(
            argstr="'-v1'",
            xor=('verbosity0', 'verbosity1', 'verbosity2'),
        ),
        verbosity2=dict(
            argstr="'v2'",
            xor=('verbosity0', 'verbosity1', 'verbosity2'),
        ),
    )
    inputs = SVReg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .diffusion import *
from .segmentation import *
from .filtering import *
from .utilities import EMSegmentTransformToNewFormat
from .surface import (MergeModels, ModelToLabelMap, GrayscaleModelMaker,
                      ProbeVolumeWithModel, LabelMapSmoothing, ModelMaker)
from .quantification import *
from .legacy import *
from .registration import *
from .converters import DicomToNrrdConverter, OrientScalarVolume



from __future__ import unicode_literals
from ..preprocess import BBRegister, BBRegisterInputSpec6


def test_BBRegister_inputs():
    input_map_5_3 = dict(
        args=dict(argstr='%s', ),
        contrast_type=dict(
            argstr='--%s',
            mandatory=True,
        ),
        dof=dict(argstr='--%d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi_mask=dict(argstr='--epi-mask', ),
        fsldof=dict(argstr='--fsl-dof %d', ),
        init=dict(
            argstr='--init-%s',
            mandatory=True,
            xor=['init_reg_file'],
        ),
        init_cost_file=dict(argstr='--initcost %s', ),
        init_reg_file=dict(
            argstr='--init-reg %s',
            mandatory=True,
            xor=['init'],
        ),
        intermediate_file=dict(argstr='--int %s', ),
        out_fsl_file=dict(argstr='--fslmat %s', ),
        out_lta_file=dict(
            argstr='--lta %s',
            min_ver='5.2.0',
        ),
        out_reg_file=dict(
            argstr='--reg %s',
            genfile=True,
        ),
        reg_frame=dict(
            argstr='--frame %d',
            xor=['reg_middle_frame'],
        ),
        reg_middle_frame=dict(
            argstr='--mid-frame',
            xor=['reg_frame'],
        ),
        registered_file=dict(argstr='--o %s', ),
        source_file=dict(
            argstr='--mov %s',
            copyfile=False,
            mandatory=True,
        ),
        spm_nifti=dict(argstr='--spm-nii', ),
        subject_id=dict(
            argstr='--s %s',
            mandatory=True,
        ),
        subjects_dir=dict(),
    )
    input_map_6_0 = dict(
        args=dict(argstr='%s', ),
        contrast_type=dict(
            argstr='--%s',
            mandatory=True,
        ),
        dof=dict(argstr='--%d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi_mask=dict(argstr='--epi-mask', ),
        fsldof=dict(argstr='--fsl-dof %d', ),
        init=dict(
            argstr='--init-%s',
            xor=['init_reg_file'],
        ),
        init_reg_file=dict(
            argstr='--init-reg %s',
            xor=['init'],
        ),
        init_cost_file=dict(argstr='--initcost %s', ),
        intermediate_file=dict(argstr='--int %s', ),
        out_fsl_file=dict(argstr='--fslmat %s', ),
        out_lta_file=dict(
            argstr='--lta %s',
            min_ver='5.2.0',
        ),
        out_reg_file=dict(
            argstr='--reg %s',
            genfile=True,
        ),
        reg_frame=dict(
            argstr='--frame %d',
            xor=['reg_middle_frame'],
        ),
        reg_middle_frame=dict(
            argstr='--mid-frame',
            xor=['reg_frame'],
        ),
        registered_file=dict(argstr='--o %s', ),
        source_file=dict(
            argstr='--mov %s',
            copyfile=False,
            mandatory=True,
        ),
        spm_nifti=dict(argstr='--spm-nii', ),
        subject_id=dict(
            argstr='--s %s',
            mandatory=True,
        ),
        subjects_dir=dict(),
    )

    instance = BBRegister()
    if isinstance(instance.inputs, BBRegisterInputSpec6):
        input_map = input_map_6_0
    else:
        input_map = input_map_5_3

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(instance.inputs.traits()[key], metakey) == value


def test_BBRegister_outputs():
    output_map = dict(
        init_cost_file=dict(),
        min_cost_file=dict(),
        out_fsl_file=dict(),
        out_lta_file=dict(),
        out_reg_file=dict(),
        registered_file=dict(),
    )
    outputs = BBRegister.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import InvWarp


def test_InvWarp_inputs():
    input_map = dict(
        absolute=dict(
            argstr='--abs',
            xor=['relative'],
        ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inverse_warp=dict(
            argstr='--out=%s',
            extensions=None,
            hash_files=False,
            name_source=['warp'],
            name_template='%s_inverse',
        ),
        jacobian_max=dict(argstr='--jmax=%f', ),
        jacobian_min=dict(argstr='--jmin=%f', ),
        niter=dict(argstr='--niter=%d', ),
        noconstraint=dict(argstr='--noconstraint', ),
        output_type=dict(),
        reference=dict(
            argstr='--ref=%s',
            extensions=None,
            mandatory=True,
        ),
        regularise=dict(argstr='--regularise=%f', ),
        relative=dict(
            argstr='--rel',
            xor=['absolute'],
        ),
        warp=dict(
            argstr='--warp=%s',
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = InvWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_InvWarp_outputs():
    output_map = dict(inverse_warp=dict(extensions=None, ), )
    outputs = InvWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import AffineRegistration


def test_AffineRegistration_inputs():
    input_map = dict(
        FixedImageFileName=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        MovingImageFileName=dict(
            argstr='%s',
            extensions=None,
            position=-1,
        ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixedsmoothingfactor=dict(argstr='--fixedsmoothingfactor %d', ),
        histogrambins=dict(argstr='--histogrambins %d', ),
        initialtransform=dict(
            argstr='--initialtransform %s',
            extensions=None,
        ),
        iterations=dict(argstr='--iterations %d', ),
        movingsmoothingfactor=dict(argstr='--movingsmoothingfactor %d', ),
        outputtransform=dict(
            argstr='--outputtransform %s',
            hash_files=False,
        ),
        resampledmovingfilename=dict(
            argstr='--resampledmovingfilename %s',
            hash_files=False,
        ),
        spatialsamples=dict(argstr='--spatialsamples %d', ),
        translationscale=dict(argstr='--translationscale %f', ),
    )
    inputs = AffineRegistration.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AffineRegistration_outputs():
    output_map = dict(
        outputtransform=dict(extensions=None, ),
        resampledmovingfilename=dict(extensions=None, ),
    )
    outputs = AffineRegistration.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""ANTS Apply Transforms interface
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import range, str
import os

from .base import ANTSCommand, ANTSCommandInputSpec
from ..base import TraitedSpec, File, traits, isdefined, InputMultiPath
from ...utils.filemanip import split_filename


class WarpTimeSeriesImageMultiTransformInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        4,
        3,
        argstr='%d',
        usedefault=True,
        desc='image dimension (3 or 4)',
        position=1)
    input_image = File(
        argstr='%s',
        mandatory=True,
        copyfile=True,
        desc=('image to apply transformation to (generally a '
              'coregistered functional)'))
    out_postfix = traits.Str(
        '_wtsimt',
        argstr='%s',
        usedefault=True,
        desc=('Postfix that is prepended to all output '
              'files (default = _wtsimt)'))
    reference_image = File(
        argstr='-R %s',
        xor=['tightest_box'],
        desc='reference image space that you wish to warp INTO')
    tightest_box = traits.Bool(
        argstr='--tightest-bounding-box',
        desc=('computes tightest bounding box (overrided by '
              'reference_image if given)'),
        xor=['reference_image'])
    reslice_by_header = traits.Bool(
        argstr='--reslice-by-header',
        desc=('Uses orientation matrix and origin encoded in '
              'reference image file header. Not typically used '
              'with additional transforms'))
    use_nearest = traits.Bool(
        argstr='--use-NN', desc='Use nearest neighbor interpolation')
    use_bspline = traits.Bool(
        argstr='--use-Bspline', desc='Use 3rd order B-Spline interpolation')
    transformation_series = InputMultiPath(
        File(exists=True),
        argstr='%s',
        desc='transformation file(s) to be applied',
        mandatory=True,
        copyfile=False)
    invert_affine = traits.List(
        traits.Int,
        desc=(
            'List of Affine transformations to invert.'
            'E.g.: [1,4,5] inverts the 1st, 4th, and 5th Affines '
            'found in transformation_series. Note that indexing '
            'starts with 1 and does not include warp fields. Affine '
            'transformations are distinguished '
            'from warp fields by the word "affine" included in their filenames.'
        ))


class WarpTimeSeriesImageMultiTransformOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='Warped image')


class WarpTimeSeriesImageMultiTransform(ANTSCommand):
    """Warps a time-series from one space to another

    Examples
    --------

    >>> from nipype.interfaces.ants import WarpTimeSeriesImageMultiTransform
    >>> wtsimt = WarpTimeSeriesImageMultiTransform()
    >>> wtsimt.inputs.input_image = 'resting.nii'
    >>> wtsimt.inputs.reference_image = 'ants_deformed.nii.gz'
    >>> wtsimt.inputs.transformation_series = ['ants_Warp.nii.gz','ants_Affine.txt']
    >>> wtsimt.cmdline
    'WarpTimeSeriesImageMultiTransform 4 resting.nii resting_wtsimt.nii -R ants_deformed.nii.gz ants_Warp.nii.gz \
ants_Affine.txt'

    >>> wtsimt = WarpTimeSeriesImageMultiTransform()
    >>> wtsimt.inputs.input_image = 'resting.nii'
    >>> wtsimt.inputs.reference_image = 'ants_deformed.nii.gz'
    >>> wtsimt.inputs.transformation_series = ['ants_Warp.nii.gz','ants_Affine.txt']
    >>> wtsimt.inputs.invert_affine = [1] # # this will invert the 1st Affine file: ants_Affine.txt
    >>> wtsimt.cmdline
    'WarpTimeSeriesImageMultiTransform 4 resting.nii resting_wtsimt.nii -R ants_deformed.nii.gz ants_Warp.nii.gz \
-i ants_Affine.txt'
    """

    _cmd = 'WarpTimeSeriesImageMultiTransform'
    input_spec = WarpTimeSeriesImageMultiTransformInputSpec
    output_spec = WarpTimeSeriesImageMultiTransformOutputSpec

    def _format_arg(self, opt, spec, val):
        if opt == 'out_postfix':
            _, name, ext = split_filename(
                os.path.abspath(self.inputs.input_image))
            return name + val + ext
        if opt == 'transformation_series':
            series = []
            affine_counter = 0
            affine_invert = []
            for transformation in val:
                if 'Affine' in transformation and \
                        isdefined(self.inputs.invert_affine):
                    affine_counter += 1
                    if affine_counter in self.inputs.invert_affine:
                        series += ['-i']
                        affine_invert.append(affine_counter)
                series += [transformation]

            if isdefined(self.inputs.invert_affine):
                diff_inv = set(self.inputs.invert_affine) - set(affine_invert)
                if diff_inv:
                    raise Exceptions(
                        "Review invert_affine, not all indexes from invert_affine were used, "
                        "check the description for the full definition")

            return ' '.join(series)
        return super(WarpTimeSeriesImageMultiTransform, self)._format_arg(
            opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        _, name, ext = split_filename(os.path.abspath(self.inputs.input_image))
        outputs['output_image'] = os.path.join(os.getcwd(), ''.join(
            (name, self.inputs.out_postfix, ext)))
        return outputs

    def _run_interface(self, runtime, correct_return_codes=[0]):
        runtime = super(WarpTimeSeriesImageMultiTransform,
                        self)._run_interface(
                            runtime, correct_return_codes=[0, 1])
        if "100 % complete" not in runtime.stdout:
            self.raise_exception(runtime)
        return runtime


class WarpImageMultiTransformInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        usedefault=True,
        desc='image dimension (2 or 3)',
        position=1)
    input_image = File(
        argstr='%s',
        mandatory=True,
        desc=('image to apply transformation to (generally a '
              'coregistered functional)'),
        position=2)
    output_image = File(
        genfile=True,
        hash_files=False,
        argstr='%s',
        desc='name of the output warped image',
        position=3,
        xor=['out_postfix'])
    out_postfix = File(
        "_wimt",
        usedefault=True,
        hash_files=False,
        desc=('Postfix that is prepended to all output '
              'files (default = _wimt)'),
        xor=['output_image'])
    reference_image = File(
        argstr='-R %s',
        xor=['tightest_box'],
        desc='reference image space that you wish to warp INTO')
    tightest_box = traits.Bool(
        argstr='--tightest-bounding-box',
        desc=('computes tightest bounding box (overrided by '
              'reference_image if given)'),
        xor=['reference_image'])
    reslice_by_header = traits.Bool(
        argstr='--reslice-by-header',
        desc=('Uses orientation matrix and origin encoded in '
              'reference image file header. Not typically used '
              'with additional transforms'))
    use_nearest = traits.Bool(
        argstr='--use-NN', desc='Use nearest neighbor interpolation')
    use_bspline = traits.Bool(
        argstr='--use-BSpline', desc='Use 3rd order B-Spline interpolation')
    transformation_series = InputMultiPath(
        File(exists=True),
        argstr='%s',
        desc='transformation file(s) to be applied',
        mandatory=True,
        position=-1)
    invert_affine = traits.List(
        traits.Int,
        desc=(
            'List of Affine transformations to invert.'
            'E.g.: [1,4,5] inverts the 1st, 4th, and 5th Affines '
            'found in transformation_series. Note that indexing '
            'starts with 1 and does not include warp fields. Affine '
            'transformations are distinguished '
            'from warp fields by the word "affine" included in their filenames.'
        ))


class WarpImageMultiTransformOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='Warped image')


class WarpImageMultiTransform(ANTSCommand):
    """Warps an image from one space to another

    Examples
    --------

    >>> from nipype.interfaces.ants import WarpImageMultiTransform
    >>> wimt = WarpImageMultiTransform()
    >>> wimt.inputs.input_image = 'structural.nii'
    >>> wimt.inputs.reference_image = 'ants_deformed.nii.gz'
    >>> wimt.inputs.transformation_series = ['ants_Warp.nii.gz','ants_Affine.txt']
    >>> wimt.cmdline
    'WarpImageMultiTransform 3 structural.nii structural_wimt.nii -R ants_deformed.nii.gz ants_Warp.nii.gz \
ants_Affine.txt'

    >>> wimt = WarpImageMultiTransform()
    >>> wimt.inputs.input_image = 'diffusion_weighted.nii'
    >>> wimt.inputs.reference_image = 'functional.nii'
    >>> wimt.inputs.transformation_series = ['func2anat_coreg_Affine.txt','func2anat_InverseWarp.nii.gz', \
    'dwi2anat_Warp.nii.gz','dwi2anat_coreg_Affine.txt']
    >>> wimt.inputs.invert_affine = [1]  # this will invert the 1st Affine file: 'func2anat_coreg_Affine.txt'
    >>> wimt.cmdline
    'WarpImageMultiTransform 3 diffusion_weighted.nii diffusion_weighted_wimt.nii -R functional.nii \
-i func2anat_coreg_Affine.txt func2anat_InverseWarp.nii.gz dwi2anat_Warp.nii.gz dwi2anat_coreg_Affine.txt'

    """

    _cmd = 'WarpImageMultiTransform'
    input_spec = WarpImageMultiTransformInputSpec
    output_spec = WarpImageMultiTransformOutputSpec

    def _gen_filename(self, name):
        if name == 'output_image':
            _, name, ext = split_filename(
                os.path.abspath(self.inputs.input_image))
            return ''.join((name, self.inputs.out_postfix, ext))
        return None

    def _format_arg(self, opt, spec, val):
        if opt == 'transformation_series':
            series = []
            affine_counter = 0
            affine_invert = []
            for transformation in val:
                if "affine" in transformation.lower() and \
                        isdefined(self.inputs.invert_affine):
                    affine_counter += 1
                    if affine_counter in self.inputs.invert_affine:
                        series += ['-i']
                        affine_invert.append(affine_counter)
                series += [transformation]

            if isdefined(self.inputs.invert_affine):
                diff_inv = set(self.inputs.invert_affine) - set(affine_invert)
                if diff_inv:
                    raise Exceptions(
                        "Review invert_affine, not all indexes from invert_affine were used, "
                        "check the description for the full definition")

            return ' '.join(series)

        return super(WarpImageMultiTransform, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.output_image):
            outputs['output_image'] = os.path.abspath(self.inputs.output_image)
        else:
            outputs['output_image'] = os.path.abspath(
                self._gen_filename('output_image'))
        return outputs


class ApplyTransformsInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        2,
        3,
        4,
        argstr='--dimensionality %d',
        desc=('This option forces the image to be treated '
              'as a specified-dimensional image. If not '
              'specified, antsWarp tries to infer the '
              'dimensionality from the input image.'))
    input_image_type = traits.Enum(
        0,
        1,
        2,
        3,
        argstr='--input-image-type %d',
        desc=('Option specifying the input image '
              'type of scalar (default), vector, '
              'tensor, or time series.'))
    input_image = File(
        argstr='--input %s',
        mandatory=True,
        desc=('image to apply transformation to (generally a '
              'coregistered functional)'),
        exists=True)
    output_image = traits.Str(
        argstr='--output %s',
        desc='output file name',
        genfile=True,
        hash_files=False)
    out_postfix = traits.Str(
        "_trans",
        usedefault=True,
        desc=('Postfix that is appended to all output '
              'files (default = _trans)'))
    reference_image = File(
        argstr='--reference-image %s',
        mandatory=True,
        desc='reference image space that you wish to warp INTO',
        exists=True)
    interpolation = traits.Enum(
        'Linear',
        'NearestNeighbor',
        'CosineWindowedSinc',
        'WelchWindowedSinc',
        'HammingWindowedSinc',
        'LanczosWindowedSinc',
        'MultiLabel',
        'Gaussian',
        'BSpline',
        argstr='%s',
        usedefault=True)
    interpolation_parameters = traits.Either(
        traits.Tuple(traits.Int()),  # BSpline (order)
        traits.Tuple(
            traits.Float(),  # Gaussian/MultiLabel (sigma, alpha)
            traits.Float()))
    transforms = traits.Either(
        InputMultiPath(File(exists=True)),
        'identity',
        argstr='%s',
        mandatory=True,
        desc='transform files: will be applied in reverse order. For '
        'example, the last specified transform will be applied first.')
    invert_transform_flags = InputMultiPath(traits.Bool())
    default_value = traits.Float(
        0.0, argstr='--default-value %g', usedefault=True)
    print_out_composite_warp_file = traits.Bool(
        False,
        requires=["output_image"],
        desc='output a composite warp file instead of a transformed image')
    float = traits.Bool(
        argstr='--float %d',
        default_value=False,
        usedefault=True,
        desc='Use float instead of double for computations.')


class ApplyTransformsOutputSpec(TraitedSpec):
    output_image = File(exists=True, desc='Warped image')


class ApplyTransforms(ANTSCommand):
    """ApplyTransforms, applied to an input image, transforms it according to a
    reference image and a transform (or a set of transforms).

    Examples
    --------

    >>> from nipype.interfaces.ants import ApplyTransforms
    >>> at = ApplyTransforms()
    >>> at.inputs.input_image = 'moving1.nii'
    >>> at.inputs.reference_image = 'fixed1.nii'
    >>> at.inputs.transforms = 'identity'
    >>> at.cmdline
    'antsApplyTransforms --default-value 0 --float 0 --input moving1.nii \
--interpolation Linear --output moving1_trans.nii \
--reference-image fixed1.nii -t identity'

    >>> at = ApplyTransforms()
    >>> at.inputs.dimension = 3
    >>> at.inputs.input_image = 'moving1.nii'
    >>> at.inputs.reference_image = 'fixed1.nii'
    >>> at.inputs.output_image = 'deformed_moving1.nii'
    >>> at.inputs.interpolation = 'Linear'
    >>> at.inputs.default_value = 0
    >>> at.inputs.transforms = ['ants_Warp.nii.gz', 'trans.mat']
    >>> at.inputs.invert_transform_flags = [False, False]
    >>> at.cmdline
    'antsApplyTransforms --default-value 0 --dimensionality 3 --float 0 --input moving1.nii \
--interpolation Linear --output deformed_moving1.nii --reference-image fixed1.nii \
--transform [ ants_Warp.nii.gz, 0 ] --transform [ trans.mat, 0 ]'

    >>> at1 = ApplyTransforms()
    >>> at1.inputs.dimension = 3
    >>> at1.inputs.input_image = 'moving1.nii'
    >>> at1.inputs.reference_image = 'fixed1.nii'
    >>> at1.inputs.output_image = 'deformed_moving1.nii'
    >>> at1.inputs.interpolation = 'BSpline'
    >>> at1.inputs.interpolation_parameters = (5,)
    >>> at1.inputs.default_value = 0
    >>> at1.inputs.transforms = ['ants_Warp.nii.gz', 'trans.mat']
    >>> at1.inputs.invert_transform_flags = [False, False]
    >>> at1.cmdline
    'antsApplyTransforms --default-value 0 --dimensionality 3 --float 0 --input moving1.nii \
--interpolation BSpline[ 5 ] --output deformed_moving1.nii --reference-image fixed1.nii \
--transform [ ants_Warp.nii.gz, 0 ] --transform [ trans.mat, 0 ]'
    """
    _cmd = 'antsApplyTransforms'
    input_spec = ApplyTransformsInputSpec
    output_spec = ApplyTransformsOutputSpec

    def _gen_filename(self, name):
        if name == 'output_image':
            output = self.inputs.output_image
            if not isdefined(output):
                _, name, ext = split_filename(self.inputs.input_image)
                output = name + self.inputs.out_postfix + ext
            return output
        return None

    def _get_transform_filenames(self):
        retval = []
        for ii in range(len(self.inputs.transforms)):
            if isdefined(self.inputs.invert_transform_flags):
                if len(self.inputs.transforms) == len(
                        self.inputs.invert_transform_flags):
                    invert_code = 1 if self.inputs.invert_transform_flags[
                        ii] else 0
                    retval.append("--transform [ %s, %d ]" %
                                  (self.inputs.transforms[ii], invert_code))
                else:
                    raise Exception((
                        "ERROR: The useInverse list must have the same number "
                        "of entries as the transformsFileName list."))
            else:
                retval.append("--transform %s" % self.inputs.transforms[ii])
        return " ".join(retval)

    def _get_output_warped_filename(self):
        if isdefined(self.inputs.print_out_composite_warp_file):
            return "--output [ %s, %d ]" % (
                self._gen_filename("output_image"),
                int(self.inputs.print_out_composite_warp_file))
        else:
            return "--output %s" % (self._gen_filename("output_image"))

    def _format_arg(self, opt, spec, val):
        if opt == "output_image":
            return self._get_output_warped_filename()
        elif opt == "transforms":
            if val == 'identity':
                return '-t identity'
            return self._get_transform_filenames()
        elif opt == 'interpolation':
            if self.inputs.interpolation in ['BSpline', 'MultiLabel', 'Gaussian'] and \
                    isdefined(self.inputs.interpolation_parameters):
                return '--interpolation %s[ %s ]' % (
                    self.inputs.interpolation, ', '.join([
                        str(param)
                        for param in self.inputs.interpolation_parameters
                    ]))
            else:
                return '--interpolation %s' % self.inputs.interpolation
        return super(ApplyTransforms, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output_image'] = os.path.abspath(
            self._gen_filename('output_image'))
        return outputs


class ApplyTransformsToPointsInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        2,
        3,
        4,
        argstr='--dimensionality %d',
        desc=('This option forces the image to be treated '
              'as a specified-dimensional image. If not '
              'specified, antsWarp tries to infer the '
              'dimensionality from the input image.'))
    input_file = File(
        argstr='--input %s',
        mandatory=True,
        desc=
        ("Currently, the only input supported is a csv file with"
         " columns including x,y (2D), x,y,z (3D) or x,y,z,t,label (4D) column headers."
         " The points should be defined in physical space."
         " If in doubt how to convert coordinates from your files to the space"
         " required by antsApplyTransformsToPoints try creating/drawing a simple"
         " label volume with only one voxel set to 1 and all others set to 0."
         " Write down the voxel coordinates. Then use ImageMaths LabelStats to find"
         " out what coordinates for this voxel antsApplyTransformsToPoints is"
         " expecting."),
        exists=True)
    output_file = traits.Str(
        argstr='--output %s',
        desc='Name of the output CSV file',
        name_source=['input_file'],
        hash_files=False,
        name_template='%s_transformed.csv')
    transforms = traits.List(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        desc='transforms that will be applied to the points')
    invert_transform_flags = traits.List(
        traits.Bool(),
        desc='list indicating if a transform should be reversed')


class ApplyTransformsToPointsOutputSpec(TraitedSpec):
    output_file = File(
        exists=True, desc='csv file with transformed coordinates')


class ApplyTransformsToPoints(ANTSCommand):
    """ApplyTransformsToPoints, applied to an CSV file, transforms coordinates
    using provided transform (or a set of transforms).

    Examples
    --------

    >>> from nipype.interfaces.ants import ApplyTransforms
    >>> at = ApplyTransformsToPoints()
    >>> at.inputs.dimension = 3
    >>> at.inputs.input_file = 'moving.csv'
    >>> at.inputs.transforms = ['trans.mat', 'ants_Warp.nii.gz']
    >>> at.inputs.invert_transform_flags = [False, False]
    >>> at.cmdline
    'antsApplyTransformsToPoints --dimensionality 3 --input moving.csv --output moving_transformed.csv \
--transform [ trans.mat, 0 ] --transform [ ants_Warp.nii.gz, 0 ]'


    """
    _cmd = 'antsApplyTransformsToPoints'
    input_spec = ApplyTransformsToPointsInputSpec
    output_spec = ApplyTransformsToPointsOutputSpec

    def _get_transform_filenames(self):
        retval = []
        for ii in range(len(self.inputs.transforms)):
            if isdefined(self.inputs.invert_transform_flags):
                if len(self.inputs.transforms) == len(
                        self.inputs.invert_transform_flags):
                    invert_code = 1 if self.inputs.invert_transform_flags[
                        ii] else 0
                    retval.append("--transform [ %s, %d ]" %
                                  (self.inputs.transforms[ii], invert_code))
                else:
                    raise Exception((
                        "ERROR: The useInverse list must have the same number "
                        "of entries as the transformsFileName list."))
            else:
                retval.append("--transform %s" % self.inputs.transforms[ii])
        return " ".join(retval)

    def _format_arg(self, opt, spec, val):
        if opt == "transforms":
            return self._get_transform_filenames()
        return super(ApplyTransformsToPoints, self)._format_arg(opt, spec, val)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..gtract import compareTractInclusion


def test_compareTractInclusion_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        closeness=dict(argstr='--closeness %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        numberOfPoints=dict(argstr='--numberOfPoints %d', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        standardFiber=dict(
            argstr='--standardFiber %s',
            extensions=None,
        ),
        testFiber=dict(
            argstr='--testFiber %s',
            extensions=None,
        ),
        testForBijection=dict(argstr='--testForBijection ', ),
        testForFiberCardinality=dict(argstr='--testForFiberCardinality ', ),
        writeXMLPolyDataFile=dict(argstr='--writeXMLPolyDataFile ', ),
    )
    inputs = compareTractInclusion.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_compareTractInclusion_outputs():
    output_map = dict()
    outputs = compareTractInclusion.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..specialized import RobustStatisticsSegmenter


def test_RobustStatisticsSegmenter_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        curvatureWeight=dict(argstr='--curvatureWeight %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        expectedVolume=dict(argstr='--expectedVolume %f', ),
        intensityHomogeneity=dict(argstr='--intensityHomogeneity %f', ),
        labelImageFileName=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        labelValue=dict(argstr='--labelValue %d', ),
        maxRunningTime=dict(argstr='--maxRunningTime %f', ),
        originalImageFileName=dict(
            argstr='%s',
            extensions=None,
            position=-3,
        ),
        segmentedImageFileName=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
    )
    inputs = RobustStatisticsSegmenter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RobustStatisticsSegmenter_outputs():
    output_map = dict(segmentedImageFileName=dict(
        extensions=None,
        position=-1,
    ), )
    outputs = RobustStatisticsSegmenter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The niftyreg module provides classes for interfacing with `niftyreg
<http://sourceforge.net/projects/niftyreg/>`_ command line tools.

These are the base tools for working with niftyreg.

Registration tools are found in niftyreg/reg.py
Every other tool is found in niftyreg/regutils.py

Examples
--------
See the docstrings of the individual classes for examples.

"""

from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import property, super
from distutils.version import StrictVersion
import os

from ... import logging
from ..base import CommandLine, CommandLineInputSpec, traits, Undefined
from ...utils.filemanip import split_filename

iflogger = logging.getLogger('nipype.interface')


def get_custom_path(command, env_dir='NIFTYREGDIR'):
    return os.path.join(os.getenv(env_dir, ''), command)


class NiftyRegCommandInputSpec(CommandLineInputSpec):
    """Input Spec for niftyreg interfaces."""
    # Set the number of omp thread to use
    omp_core_val = traits.Int(
        int(os.environ.get('OMP_NUM_THREADS', '1')),
        desc='Number of openmp thread to use',
        argstr='-omp %i',
        usedefault=True)


class NiftyRegCommand(CommandLine):
    """
    Base support interface for NiftyReg commands.
    """
    _suffix = '_nr'
    _min_version = '1.5.30'

    input_spec = NiftyRegCommandInputSpec

    def __init__(self, required_version=None, **inputs):
        self.num_threads = 1
        super(NiftyRegCommand, self).__init__(**inputs)
        self.required_version = required_version
        _version = self.version_from_command()
        if _version:
            _version = _version.decode("utf-8")
            if self._min_version is not None and \
               StrictVersion(_version) < StrictVersion(self._min_version):
                msg = 'A later version of Niftyreg is required (%s < %s)'
                iflogger.warning(msg, _version, self._min_version)
            if required_version is not None:
                if StrictVersion(_version) != StrictVersion(required_version):
                    msg = 'The version of NiftyReg differs from the required'
                    msg += '(%s != %s)'
                    iflogger.warning(msg, _version, self.required_version)
        self.inputs.on_trait_change(self._omp_update, 'omp_core_val')
        self.inputs.on_trait_change(self._environ_update, 'environ')
        self._omp_update()

    def _omp_update(self):
        if self.inputs.omp_core_val:
            self.inputs.environ['OMP_NUM_THREADS'] = \
                str(self.inputs.omp_core_val)
            self.num_threads = self.inputs.omp_core_val
        else:
            if 'OMP_NUM_THREADS' in self.inputs.environ:
                del self.inputs.environ['OMP_NUM_THREADS']
            self.num_threads = 1

    def _environ_update(self):
        if self.inputs.environ:
            if 'OMP_NUM_THREADS' in self.inputs.environ:
                self.inputs.omp_core_val = \
                    int(self.inputs.environ['OMP_NUM_THREADS'])
            else:
                self.inputs.omp_core_val = Undefined
        else:
            self.inputs.omp_core_val = Undefined

    def check_version(self):
        _version = self.version_from_command()
        if not _version:
            raise Exception('Niftyreg not found')
        # Decoding to string:
        _version = _version.decode("utf-8")
        if StrictVersion(_version) < StrictVersion(self._min_version):
            err = 'A later version of Niftyreg is required (%s < %s)'
            raise ValueError(err % (_version, self._min_version))
        if self.required_version:
            if StrictVersion(_version) != StrictVersion(self.required_version):
                err = 'The version of NiftyReg differs from the required'
                err += '(%s != %s)'
                raise ValueError(err % (_version, self.required_version))

    @property
    def version(self):
        return self.version_from_command()

    def exists(self):
        return self.version_from_command() is not None

    def _format_arg(self, name, spec, value):
        if name == 'omp_core_val':
            self.numthreads = value
        return super(NiftyRegCommand, self)._format_arg(name, spec, value)

    def _gen_fname(self, basename, out_dir=None, suffix=None, ext=None):
        if basename == '':
            msg = 'Unable to generate filename for command %s. ' % self.cmd
            msg += 'basename is not set!'
            raise ValueError(msg)
        _, final_bn, final_ext = split_filename(basename)
        if out_dir is None:
            out_dir = os.getcwd()
        if ext is not None:
            final_ext = ext
        if suffix is not None:
            final_bn = ''.join((final_bn, suffix))
        return os.path.abspath(os.path.join(out_dir, final_bn + final_ext))



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import GCOR


def test_GCOR_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-input %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        mask=dict(
            argstr='-mask %s',
            copyfile=False,
            extensions=None,
        ),
        nfirst=dict(argstr='-nfirst %d', ),
        no_demean=dict(argstr='-no_demean', ),
    )
    inputs = GCOR.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_GCOR_outputs():
    output_map = dict(out=dict(), )
    outputs = GCOR.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import ClipLevel


def test_ClipLevel_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        doall=dict(
            argstr='-doall',
            position=3,
            xor='grad',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        grad=dict(
            argstr='-grad %s',
            extensions=None,
            position=3,
            xor='doall',
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        mfrac=dict(
            argstr='-mfrac %s',
            position=2,
        ),
    )
    inputs = ClipLevel.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ClipLevel_outputs():
    output_map = dict(clip_val=dict(), )
    outputs = ClipLevel.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import nipype.interfaces.fsl as fsl
from nipype.interfaces.base import InterfaceResult
from nipype.interfaces.fsl import check_fsl, no_fsl

import pytest


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fslversion():
    ver = fsl.Info.version()
    ver = ver.split('.')
    assert ver[0] in ['4', '5']


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fsloutputtype():
    types = list(fsl.Info.ftypes.keys())
    orig_out_type = fsl.Info.output_type()
    assert orig_out_type in types


def test_outputtype_to_ext():
    for ftype, ext in fsl.Info.ftypes.items():
        res = fsl.Info.output_type_to_ext(ftype)
        assert res == ext

    with pytest.raises(KeyError):
        fsl.Info.output_type_to_ext('JUNK')


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_FSLCommand():
    # Most methods in FSLCommand are tested in the subclasses.  Only
    # testing the one item that is not.
    cmd = fsl.FSLCommand(command='ls')
    res = cmd.run()
    assert type(res) == InterfaceResult


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_FSLCommand2():
    # Check default output type and environ
    cmd = fsl.FSLCommand(command='junk')
    assert cmd._output_type == fsl.Info.output_type()
    assert cmd.inputs.environ['FSLOUTPUTTYPE'] == cmd._output_type
    assert cmd._output_type in fsl.Info.ftypes

    cmd = fsl.FSLCommand
    cmdinst = fsl.FSLCommand(command='junk')
    for out_type in fsl.Info.ftypes:
        cmd.set_default_output_type(out_type)
        assert cmd._output_type == out_type
        if out_type != fsl.Info.output_type():
            #  Setting class outputtype should not effect existing instances
            assert cmdinst.inputs.output_type != out_type


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
@pytest.mark.parametrize(
    "args, desired_name",
    [
        ({}, {
            "file": 'foo.nii.gz'
        }),  # just the filename
        # filename with suffix
        ({
            "suffix": '_brain'
        }, {
            "file": 'foo_brain.nii.gz'
        }),
        (
            {
                "suffix": '_brain',
                "cwd": '/data'
            },
            # filename with suffix and working directory
            {
                "dir": '/data',
                "file": 'foo_brain.nii.gz'
            }),
        # filename with suffix and no file extension change
        ({
            "suffix": '_brain.mat',
            "change_ext": False
        }, {
            "file": 'foo_brain.mat'
        })
    ])
def test_gen_fname(args, desired_name):
    # Test _gen_fname method of FSLCommand
    cmd = fsl.FSLCommand(command='junk', output_type='NIFTI_GZ')
    pth = os.getcwd()
    fname = cmd._gen_fname('foo.nii.gz', **args)
    if "dir" in desired_name.keys():
        desired = os.path.join(desired_name["dir"], desired_name["file"])
    else:
        desired = os.path.join(pth, desired_name["file"])
    assert fname == desired



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os.path as op

import numpy as np
import networkx as nx

from ... import logging
from ..base import (LibraryBaseInterface, BaseInterfaceInputSpec, traits, File,
                    TraitedSpec, InputMultiPath, OutputMultiPath, isdefined)
from .base import have_cv
iflogger = logging.getLogger('nipype.interface')


def ntwks_to_matrices(in_files, edge_key):
    first = nx.read_gpickle(in_files[0])
    files = len(in_files)
    nodes = len(first.nodes())
    matrix = np.zeros((nodes, nodes, files))
    for idx, name in enumerate(in_files):
        graph = nx.read_gpickle(name)
        for u, v, d in graph.edges(data=True):
            try:
                graph[u][v]['weight'] = d[
                    edge_key]  # Setting the edge requested edge value as weight value
            except:
                raise KeyError(
                    "the graph edges do not have {} attribute".format(
                        edge_key))
        matrix[:, :, idx] = nx.to_numpy_matrix(graph)  # Retrieve the matrix
    return matrix


class NetworkBasedStatisticInputSpec(BaseInterfaceInputSpec):
    in_group1 = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc='Networks for the first group of subjects')
    in_group2 = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc='Networks for the second group of subjects')
    node_position_network = File(
        desc=
        'An optional network used to position the nodes for the output networks'
    )
    number_of_permutations = traits.Int(
        1000, usedefault=True, desc='Number of permutations to perform')
    threshold = traits.Float(3, usedefault=True, desc='T-statistic threshold')
    t_tail = traits.Enum(
        'left',
        'right',
        'both',
        usedefault=True,
        desc='Can be one of "left", "right", or "both"')
    edge_key = traits.Str(
        'number_of_fibers',
        usedefault=True,
        desc=
        'Usually "number_of_fibers, "fiber_length_mean", "fiber_length_std" for matrices made with CMTK'
        'Sometimes "weight" or "value" for functional networks.')
    out_nbs_network = File(
        desc='Output network with edges identified by the NBS')
    out_nbs_pval_network = File(
        desc=
        'Output network with p-values to weight the edges identified by the NBS'
    )


class NetworkBasedStatisticOutputSpec(TraitedSpec):
    nbs_network = File(
        exists=True, desc='Output network with edges identified by the NBS')
    nbs_pval_network = File(
        exists=True,
        desc=
        'Output network with p-values to weight the edges identified by the NBS'
    )
    network_files = OutputMultiPath(
        File(exists=True),
        desc='Output network with edges identified by the NBS')


class NetworkBasedStatistic(LibraryBaseInterface):
    """
    Calculates and outputs the average network given a set of input NetworkX gpickle files

    For documentation of Network-based statistic parameters:

            https://github.com/LTS5/connectomeviewer/blob/master/cviewer/libs/pyconto/groupstatistics/nbs/_nbs.py

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> nbs = cmtk.NetworkBasedStatistic()
    >>> nbs.inputs.in_group1 = ['subj1.pck', 'subj2.pck'] # doctest: +SKIP
    >>> nbs.inputs.in_group2 = ['pat1.pck', 'pat2.pck'] # doctest: +SKIP
    >>> nbs.run()                 # doctest: +SKIP
    """
    input_spec = NetworkBasedStatisticInputSpec
    output_spec = NetworkBasedStatisticOutputSpec
    _pkg = 'cviewer'

    def _run_interface(self, runtime):
        from cviewer.libs.pyconto.groupstatistics import nbs

        THRESH = self.inputs.threshold
        K = self.inputs.number_of_permutations
        TAIL = self.inputs.t_tail
        edge_key = self.inputs.edge_key
        details = edge_key + '-thresh-' + str(THRESH) + '-k-' + str(
            K) + '-tail-' + TAIL + '.pck'

        # Fill in the data from the networks
        X = ntwks_to_matrices(self.inputs.in_group1, edge_key)
        Y = ntwks_to_matrices(self.inputs.in_group2, edge_key)

        PVAL, ADJ, _ = nbs.compute_nbs(X, Y, THRESH, K, TAIL)

        iflogger.info('p-values:')
        iflogger.info(PVAL)

        pADJ = ADJ.copy()
        for idx, _ in enumerate(PVAL):
            x, y = np.where(ADJ == idx + 1)
            pADJ[x, y] = PVAL[idx]

        # Create networkx graphs from the adjacency matrix
        nbsgraph = nx.from_numpy_matrix(ADJ)
        nbs_pval_graph = nx.from_numpy_matrix(pADJ)

        # Relabel nodes because they should not start at zero for our convention
        nbsgraph = nx.relabel_nodes(nbsgraph, lambda x: x + 1)
        nbs_pval_graph = nx.relabel_nodes(nbs_pval_graph, lambda x: x + 1)

        if isdefined(self.inputs.node_position_network):
            node_ntwk_name = self.inputs.node_position_network
        else:
            node_ntwk_name = self.inputs.in_group1[0]

        node_network = nx.read_gpickle(node_ntwk_name)
        iflogger.info('Populating node dictionaries with attributes from %s',
                      node_ntwk_name)

        for nid, ndata in node_network.nodes(data=True):
            nbsgraph.nodes[nid] = ndata
            nbs_pval_graph.nodes[nid] = ndata

        path = op.abspath('NBS_Result_' + details)
        iflogger.info(path)
        nx.write_gpickle(nbsgraph, path)
        iflogger.info('Saving output NBS edge network as %s', path)

        pval_path = op.abspath('NBS_P_vals_' + details)
        iflogger.info(pval_path)
        nx.write_gpickle(nbs_pval_graph, pval_path)
        iflogger.info('Saving output p-value network as %s', pval_path)
        return runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()

        THRESH = self.inputs.threshold
        K = self.inputs.number_of_permutations
        TAIL = self.inputs.t_tail
        edge_key = self.inputs.edge_key
        details = edge_key + '-thresh-' + str(THRESH) + '-k-' + str(
            K) + '-tail-' + TAIL + '.pck'
        path = op.abspath('NBS_Result_' + details)
        pval_path = op.abspath('NBS_P_vals_' + details)

        outputs['nbs_network'] = path
        outputs['nbs_pval_network'] = pval_path
        outputs['network_files'] = [path, pval_path]
        return outputs

    def _gen_outfilename(self, name, ext):
        return name + '.' + ext



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Aparc2Aseg


def test_Aparc2Aseg_inputs():
    input_map = dict(
        a2009s=dict(argstr='--a2009s', ),
        args=dict(argstr='%s', ),
        aseg=dict(
            argstr='--aseg %s',
            extensions=None,
        ),
        copy_inputs=dict(),
        ctxseg=dict(
            argstr='--ctxseg %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filled=dict(extensions=None, ),
        hypo_wm=dict(argstr='--hypo-as-wm', ),
        label_wm=dict(argstr='--labelwm', ),
        lh_annotation=dict(
            extensions=None,
            mandatory=True,
        ),
        lh_pial=dict(
            extensions=None,
            mandatory=True,
        ),
        lh_ribbon=dict(
            extensions=None,
            mandatory=True,
        ),
        lh_white=dict(
            extensions=None,
            mandatory=True,
        ),
        out_file=dict(
            argstr='--o %s',
            extensions=None,
            mandatory=True,
        ),
        rh_annotation=dict(
            extensions=None,
            mandatory=True,
        ),
        rh_pial=dict(
            extensions=None,
            mandatory=True,
        ),
        rh_ribbon=dict(
            extensions=None,
            mandatory=True,
        ),
        rh_white=dict(
            extensions=None,
            mandatory=True,
        ),
        ribbon=dict(
            extensions=None,
            mandatory=True,
        ),
        rip_unknown=dict(argstr='--rip-unknown', ),
        subject_id=dict(
            argstr='--s %s',
            mandatory=True,
            usedefault=True,
        ),
        subjects_dir=dict(),
        volmask=dict(argstr='--volmask', ),
    )
    inputs = Aparc2Aseg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Aparc2Aseg_outputs():
    output_map = dict(out_file=dict(
        argstr='%s',
        extensions=None,
    ), )
    outputs = Aparc2Aseg.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..convert import FSL2Scheme


def test_FSL2Scheme_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bscale=dict(
            argstr='-bscale %d',
            units='NA',
        ),
        bval_file=dict(
            argstr='-bvalfile %s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        bvec_file=dict(
            argstr='-bvecfile %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        diffusiontime=dict(
            argstr='-diffusiontime %f',
            units='NA',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        flipx=dict(argstr='-flipx', ),
        flipy=dict(argstr='-flipy', ),
        flipz=dict(argstr='-flipz', ),
        interleave=dict(argstr='-interleave', ),
        numscans=dict(
            argstr='-numscans %d',
            units='NA',
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        usegradmod=dict(argstr='-usegradmod', ),
    )
    inputs = FSL2Scheme.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FSL2Scheme_outputs():
    output_map = dict(scheme=dict(extensions=None, ), )
    outputs = FSL2Scheme.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..segmentation import KellyKapowski


def test_KellyKapowski_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        convergence=dict(
            argstr='--convergence "%s"',
            usedefault=True,
        ),
        cortical_thickness=dict(
            argstr='--output "%s"',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=['segmentation_image'],
            name_template='%s_cortical_thickness',
        ),
        dimension=dict(
            argstr='--image-dimensionality %d',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        gradient_step=dict(
            argstr='--gradient-step %f',
            usedefault=True,
        ),
        gray_matter_label=dict(usedefault=True, ),
        gray_matter_prob_image=dict(
            argstr='--gray-matter-probability-image "%s"',
            extensions=None,
        ),
        max_invert_displacement_field_iters=dict(
            argstr=
            '--maximum-number-of-invert-displacement-field-iterations %d',
            usedefault=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        number_integration_points=dict(
            argstr='--number-of-integration-points %d',
            usedefault=True,
        ),
        segmentation_image=dict(
            argstr='--segmentation-image "%s"',
            extensions=None,
            mandatory=True,
        ),
        smoothing_variance=dict(
            argstr='--smoothing-variance %f',
            usedefault=True,
        ),
        smoothing_velocity_field=dict(
            argstr='--smoothing-velocity-field-parameter %f',
            usedefault=True,
        ),
        thickness_prior_estimate=dict(
            argstr='--thickness-prior-estimate %f',
            usedefault=True,
        ),
        thickness_prior_image=dict(
            argstr='--thickness-prior-image "%s"',
            extensions=None,
        ),
        use_bspline_smoothing=dict(argstr='--use-bspline-smoothing 1', ),
        warped_white_matter=dict(
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=['segmentation_image'],
            name_template='%s_warped_white_matter',
        ),
        white_matter_label=dict(usedefault=True, ),
        white_matter_prob_image=dict(
            argstr='--white-matter-probability-image "%s"',
            extensions=None,
        ),
    )
    inputs = KellyKapowski.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_KellyKapowski_outputs():
    output_map = dict(
        cortical_thickness=dict(extensions=None, ),
        warped_white_matter=dict(extensions=None, ),
    )
    outputs = KellyKapowski.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..fibertrack import fibertrack


def test_fibertrack_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        forbidden_label=dict(argstr='--forbidden_label %d', ),
        force=dict(argstr='--force ', ),
        input_roi_file=dict(
            argstr='--input_roi_file %s',
            extensions=None,
        ),
        input_tensor_file=dict(
            argstr='--input_tensor_file %s',
            extensions=None,
        ),
        max_angle=dict(argstr='--max_angle %f', ),
        min_fa=dict(argstr='--min_fa %f', ),
        output_fiber_file=dict(
            argstr='--output_fiber_file %s',
            hash_files=False,
        ),
        really_verbose=dict(argstr='--really_verbose ', ),
        source_label=dict(argstr='--source_label %d', ),
        step_size=dict(argstr='--step_size %f', ),
        target_label=dict(argstr='--target_label %d', ),
        verbose=dict(argstr='--verbose ', ),
        whole_brain=dict(argstr='--whole_brain ', ),
    )
    inputs = fibertrack.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_fibertrack_outputs():
    output_map = dict(output_fiber_file=dict(extensions=None, ), )
    outputs = fibertrack.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Defines functionality for pipelined execution of interfaces

The `EngineBase` class implements the more general view of a task.
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import object

from copy import deepcopy
import re
import numpy as np

from ... import config
from ...interfaces.base import DynamicTraitedSpec
from ...utils.filemanip import loadpkl, savepkl


class EngineBase(object):
    """Defines common attributes and functions for workflows and nodes."""

    def __init__(self, name=None, base_dir=None):
        """ Initialize base parameters of a workflow or node

        Parameters
        ----------
        name : string (mandatory)
            Name of this node. Name must be alphanumeric and not contain any
            special characters (e.g., '.', '@').
        base_dir : string
            base output directory (will be hashed before creations)
            default=None, which results in the use of mkdtemp

        """
        self._hierarchy = None
        self.name = name
        self._id = self.name # for compatibility with node expansion using iterables

        self.base_dir = base_dir
        self.config = deepcopy(config._sections)

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, name):
        if not name or not re.match(r'^[\w-]+$', name):
            raise ValueError('[Workflow|Node] name "%s" is not valid.' % name)
        self._name = name

    @property
    def fullname(self):
        if self._hierarchy:
            return '%s.%s' % (self._hierarchy, self.name)
        return self.name

    @property
    def inputs(self):
        raise NotImplementedError

    @property
    def outputs(self):
        raise NotImplementedError

    @property
    def itername(self):
        """Name for expanded iterable"""
        itername = self._id
        if self._hierarchy:
            itername = '%s.%s' % (self._hierarchy, self._id)
        return itername

    def clone(self, name):
        """Clone an EngineBase object

        Parameters
        ----------

        name : string (mandatory)
            A clone of node or workflow must have a new name
        """
        if name == self.name:
            raise ValueError('Cloning requires a new name, "%s" is '
                             'in use.' % name)
        clone = deepcopy(self)
        clone.name = name
        if hasattr(clone, '_id'):
            clone._id = name
        return clone

    def _check_outputs(self, parameter):
        return hasattr(self.outputs, parameter)

    def _check_inputs(self, parameter):
        if isinstance(self.inputs, DynamicTraitedSpec):
            return True
        return hasattr(self.inputs, parameter)

    def __str__(self):
        return self.fullname

    def __repr__(self):
        return self.itername

    def save(self, filename=None):
        if filename is None:
            filename = 'temp.pklz'
        savepkl(filename, self)

    def load(self, filename):
        return loadpkl(filename)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import str, open, range

import os.path as op
import pickle

import numpy as np
import networkx as nx

from ... import logging
from ...utils.filemanip import split_filename
from ..base import (BaseInterface, BaseInterfaceInputSpec, traits, File,
                    TraitedSpec, InputMultiPath, OutputMultiPath, isdefined)
from .base import have_cmp

iflogger = logging.getLogger('nipype.interface')


def read_unknown_ntwk(ntwk):
    if not isinstance(ntwk, nx.classes.graph.Graph):
        _, _, ext = split_filename(ntwk)
        if ext == '.pck':
            ntwk = nx.read_gpickle(ntwk)
        elif ext == '.graphml':
            ntwk = nx.read_graphml(ntwk)
    return ntwk


def remove_all_edges(ntwk):
    ntwktmp = ntwk.copy()
    edges = list(ntwktmp.edges())
    for edge in edges:
        ntwk.remove_edge(edge[0], edge[1])
    return ntwk


def fix_keys_for_gexf(orig):
    """
    GEXF Networks can be read in Gephi, however, the keys for the node and edge IDs must be converted to strings
    """
    import networkx as nx
    ntwk = nx.Graph()
    nodes = list(orig.nodes())
    edges = list(orig.edges())
    for node in nodes:
        newnodedata = {}
        newnodedata.update(orig.nodes[node])
        if 'dn_fsname' in orig.nodes[node]:
            newnodedata['label'] = orig.nodes[node]['dn_fsname']
        ntwk.add_node(str(node), **newnodedata)
        if 'dn_position' in ntwk.nodes[str(
                node)] and 'dn_position' in newnodedata:
            ntwk.nodes[str(node)]['dn_position'] = str(
                newnodedata['dn_position'])
    for edge in edges:
        data = {}
        data = orig.edge[edge[0]][edge[1]]
        ntwk.add_edge(str(edge[0]), str(edge[1]), **data)
        if 'fiber_length_mean' in ntwk.edge[str(edge[0])][str(edge[1])]:
            ntwk.edge[str(edge[0])][str(edge[1])]['fiber_length_mean'] = str(
                data['fiber_length_mean'])
        if 'fiber_length_std' in ntwk.edge[str(edge[0])][str(edge[1])]:
            ntwk.edge[str(edge[0])][str(edge[1])]['fiber_length_std'] = str(
                data['fiber_length_std'])
        if 'number_of_fibers' in ntwk.edge[str(edge[0])][str(edge[1])]:
            ntwk.edge[str(edge[0])][str(edge[1])]['number_of_fibers'] = str(
                data['number_of_fibers'])
        if 'value' in ntwk.edge[str(edge[0])][str(edge[1])]:
            ntwk.edge[str(edge[0])][str(edge[1])]['value'] = str(data['value'])
    return ntwk


def add_dicts_by_key(in_dict1, in_dict2):
    """
    Combines two dictionaries and adds the values for those keys that are shared
    """
    both = {}
    for key1 in in_dict1:
        for key2 in in_dict2:
            if key1 == key2:
                both[key1] = in_dict1[key1] + in_dict2[key2]
    return both


def average_networks(in_files, ntwk_res_file, group_id):
    """
    Sums the edges of input networks and divides by the number of networks
    Writes the average network as .pck and .gexf and returns the name of the written networks
    """
    import networkx as nx
    import os.path as op
    import scipy.io as sio
    iflogger.info('Creating average network for group: %s', group_id)
    matlab_network_list = []
    if len(in_files) == 1:
        avg_ntwk = read_unknown_ntwk(in_files[0])
    else:
        count_to_keep_edge = np.round(len(in_files) / 2.0)
        iflogger.info('Number of networks: %i, an edge must occur in at '
                      'least %i to remain in the average network',
                      len(in_files), count_to_keep_edge)
        ntwk_res_file = read_unknown_ntwk(ntwk_res_file)
        iflogger.info('%i nodes found in network resolution file',
                      ntwk_res_file.number_of_nodes())
        ntwk = remove_all_edges(ntwk_res_file)
        counting_ntwk = ntwk.copy()
        # Sums all the relevant variables
        for index, subject in enumerate(in_files):
            tmp = nx.read_gpickle(subject)
            iflogger.info('File %s has %i edges', subject,
                          tmp.number_of_edges())
            edges = list(tmp.edges())
            for edge in edges:
                data = {}
                data = tmp.edge[edge[0]][edge[1]]
                data['count'] = 1
                if ntwk.has_edge(edge[0], edge[1]):
                    current = {}
                    current = ntwk.edge[edge[0]][edge[1]]
                    data = add_dicts_by_key(current, data)
                ntwk.add_edge(edge[0], edge[1], **data)
            nodes = list(tmp.nodes())
            for node in nodes:
                data = {}
                data = ntwk.nodes[node]
                if 'value' in tmp.nodes[node]:
                    data['value'] = data['value'] + tmp.nodes[node]['value']
                ntwk.add_node(node, **data)

        # Divides each value by the number of files
        nodes = list(ntwk.nodes())
        edges = list(ntwk.edges())
        iflogger.info('Total network has %i edges', ntwk.number_of_edges())
        avg_ntwk = nx.Graph()
        newdata = {}
        for node in nodes:
            data = ntwk.nodes[node]
            newdata = data
            if 'value' in data:
                newdata['value'] = data['value'] / len(in_files)
                ntwk.nodes[node]['value'] = newdata
            avg_ntwk.add_node(node, **newdata)

        edge_dict = {}
        edge_dict['count'] = np.zeros((avg_ntwk.number_of_nodes(),
                                       avg_ntwk.number_of_nodes()))
        for edge in edges:
            data = ntwk.edge[edge[0]][edge[1]]
            if ntwk.edge[edge[0]][edge[1]]['count'] >= count_to_keep_edge:
                for key in list(data.keys()):
                    if not key == 'count':
                        data[key] = data[key] / len(in_files)
                ntwk.edge[edge[0]][edge[1]] = data
                avg_ntwk.add_edge(edge[0], edge[1], **data)
            edge_dict['count'][edge[0] - 1][edge[1] - 1] = ntwk.edge[edge[0]][
                edge[1]]['count']

        iflogger.info('After thresholding, the average network has %i edges',
                      avg_ntwk.number_of_edges())

        avg_edges = avg_ntwk.edges()
        for edge in avg_edges:
            data = avg_ntwk.edge[edge[0]][edge[1]]
            for key in list(data.keys()):
                if not key == 'count':
                    edge_dict[key] = np.zeros((avg_ntwk.number_of_nodes(),
                                               avg_ntwk.number_of_nodes()))
                    edge_dict[key][edge[0] - 1][edge[1] - 1] = data[key]

        for key in list(edge_dict.keys()):
            tmp = {}
            network_name = group_id + '_' + key + '_average.mat'
            matlab_network_list.append(op.abspath(network_name))
            tmp[key] = edge_dict[key]
            sio.savemat(op.abspath(network_name), tmp)
            iflogger.info('Saving average network for key: %s as %s', key,
                          op.abspath(network_name))

    # Writes the networks and returns the name
    network_name = group_id + '_average.pck'
    nx.write_gpickle(avg_ntwk, op.abspath(network_name))
    iflogger.info('Saving average network as %s', op.abspath(network_name))
    avg_ntwk = fix_keys_for_gexf(avg_ntwk)
    network_name = group_id + '_average.gexf'
    nx.write_gexf(avg_ntwk, op.abspath(network_name))
    iflogger.info('Saving average network as %s', op.abspath(network_name))
    return network_name, matlab_network_list


def compute_node_measures(ntwk, calculate_cliques=False):
    """
    These return node-based measures
    """
    iflogger.info('Computing node measures:')
    measures = {}
    iflogger.info('...Computing degree...')
    measures['degree'] = np.array(list(ntwk.degree().values()))
    iflogger.info('...Computing load centrality...')
    measures['load_centrality'] = np.array(
        list(nx.load_centrality(ntwk).values()))
    iflogger.info('...Computing betweenness centrality...')
    measures['betweenness_centrality'] = np.array(
        list(nx.betweenness_centrality(ntwk).values()))
    iflogger.info('...Computing degree centrality...')
    measures['degree_centrality'] = np.array(
        list(nx.degree_centrality(ntwk).values()))
    iflogger.info('...Computing closeness centrality...')
    measures['closeness_centrality'] = np.array(
        list(nx.closeness_centrality(ntwk).values()))
    #    iflogger.info('...Computing eigenvector centrality...')
    #    measures['eigenvector_centrality'] = np.array(nx.eigenvector_centrality(ntwk, max_iter=100000).values())
    iflogger.info('...Computing triangles...')
    measures['triangles'] = np.array(list(nx.triangles(ntwk).values()))
    iflogger.info('...Computing clustering...')
    measures['clustering'] = np.array(list(nx.clustering(ntwk).values()))
    iflogger.info('...Computing k-core number')
    measures['core_number'] = np.array(list(nx.core_number(ntwk).values()))
    iflogger.info('...Identifying network isolates...')
    isolate_list = nx.isolates(ntwk)
    binarized = np.zeros((ntwk.number_of_nodes(), 1))
    for value in isolate_list:
        value = value - 1  # Zero indexing
        binarized[value] = 1
    measures['isolates'] = binarized
    if calculate_cliques:
        iflogger.info('...Calculating node clique number')
        measures['node_clique_number'] = np.array(
            list(nx.node_clique_number(ntwk).values()))
        iflogger.info('...Computing number of cliques for each node...')
        measures['number_of_cliques'] = np.array(
            list(nx.number_of_cliques(ntwk).values()))
    return measures


def compute_edge_measures(ntwk):
    """
    These return edge-based measures
    """
    iflogger.info('Computing edge measures:')
    measures = {}
    # iflogger.info('...Computing google matrix...' #Makes really large networks (500k+ edges))
    # measures['google_matrix'] = nx.google_matrix(ntwk)
    # iflogger.info('...Computing hub matrix...')
    # measures['hub_matrix'] = nx.hub_matrix(ntwk)
    # iflogger.info('...Computing authority matrix...')
    # measures['authority_matrix'] = nx.authority_matrix(ntwk)
    return measures


def compute_dict_measures(ntwk):
    """
    Returns a dictionary
    """
    iflogger.info('Computing measures which return a dictionary:')
    measures = {}
    iflogger.info('...Computing rich club coefficient...')
    measures['rich_club_coef'] = nx.rich_club_coefficient(ntwk)
    return measures


def compute_singlevalued_measures(ntwk, weighted=True,
                                  calculate_cliques=False):
    """
    Returns a single value per network
    """
    iflogger.info('Computing single valued measures:')
    measures = {}
    iflogger.info('...Computing degree assortativity (pearson number) ...')
    try:
        measures['degree_pearsonr'] = nx.degree_pearsonr(ntwk)
    except AttributeError:  # For NetworkX 1.6
        measures[
            'degree_pearsonr'] = nx.degree_pearson_correlation_coefficient(
                ntwk)
    iflogger.info('...Computing degree assortativity...')
    try:
        measures['degree_assortativity'] = nx.degree_assortativity(ntwk)
    except AttributeError:
        measures['degree_assortativity'] = nx.degree_assortativity_coefficient(
            ntwk)
    iflogger.info('...Computing transitivity...')
    measures['transitivity'] = nx.transitivity(ntwk)
    iflogger.info('...Computing number of connected_components...')
    measures['number_connected_components'] = nx.number_connected_components(
        ntwk)
    iflogger.info('...Computing graph density...')
    measures['graph_density'] = nx.density(ntwk)
    iflogger.info('...Recording number of edges...')
    measures['number_of_edges'] = nx.number_of_edges(ntwk)
    iflogger.info('...Recording number of nodes...')
    measures['number_of_nodes'] = nx.number_of_nodes(ntwk)
    iflogger.info('...Computing average clustering...')
    measures['average_clustering'] = nx.average_clustering(ntwk)
    if nx.is_connected(ntwk):
        iflogger.info('...Calculating average shortest path length...')
        measures[
            'average_shortest_path_length'] = nx.average_shortest_path_length(
                ntwk, weighted)
    else:
        iflogger.info('...Calculating average shortest path length...')
        measures[
            'average_shortest_path_length'] = nx.average_shortest_path_length(
                nx.connected_component_subgraphs(ntwk)[0], weighted)
    if calculate_cliques:
        iflogger.info('...Computing graph clique number...')
        measures['graph_clique_number'] = nx.graph_clique_number(
            ntwk)  # out of memory error
    return measures


def compute_network_measures(ntwk):
    measures = {}
    # iflogger.info('Identifying k-core')
    # measures['k_core'] = nx.k_core(ntwk)
    # iflogger.info('Identifying k-shell')
    # measures['k_shell'] = nx.k_shell(ntwk)
    # iflogger.info('Identifying k-crust')
    # measures['k_crust'] = nx.k_crust(ntwk)
    return measures


def add_node_data(node_array, ntwk):
    node_ntwk = nx.Graph()
    newdata = {}
    for idx, data in ntwk.nodes(data=True):
        if not int(idx) == 0:
            newdata['value'] = node_array[int(idx) - 1]
            data.update(newdata)
            node_ntwk.add_node(int(idx), **data)
    return node_ntwk


def add_edge_data(edge_array, ntwk, above=0, below=0):
    edge_ntwk = ntwk.copy()
    data = {}
    for x, row in enumerate(edge_array):
        for y in range(0, np.max(np.shape(edge_array[x]))):
            if not edge_array[x, y] == 0:
                data['value'] = edge_array[x, y]
                if data['value'] <= below or data['value'] >= above:
                    if edge_ntwk.has_edge(x + 1, y + 1):
                        old_edge_dict = edge_ntwk.edge[x + 1][y + 1]
                        edge_ntwk.remove_edge(x + 1, y + 1)
                        data.update(old_edge_dict)
                    edge_ntwk.add_edge(x + 1, y + 1, **data)
    return edge_ntwk


class NetworkXMetricsInputSpec(BaseInterfaceInputSpec):
    in_file = File(exists=True, mandatory=True, desc='Input network')
    out_k_core = File(
        'k_core',
        usedefault=True,
        desc='Computed k-core network stored as a NetworkX pickle.')
    out_k_shell = File(
        'k_shell',
        usedefault=True,
        desc='Computed k-shell network stored as a NetworkX pickle.')
    out_k_crust = File(
        'k_crust',
        usedefault=True,
        desc='Computed k-crust network stored as a NetworkX pickle.')
    treat_as_weighted_graph = traits.Bool(
        True,
        usedefault=True,
        desc=
        'Some network metrics can be calculated while considering only a binarized version of the graph'
    )
    compute_clique_related_measures = traits.Bool(
        False,
        usedefault=True,
        desc=
        'Computing clique-related measures (e.g. node clique number) can be very time consuming'
    )
    out_global_metrics_matlab = File(
        genfile=True, desc='Output node metrics in MATLAB .mat format')
    out_node_metrics_matlab = File(
        genfile=True, desc='Output node metrics in MATLAB .mat format')
    out_edge_metrics_matlab = File(
        genfile=True, desc='Output edge metrics in MATLAB .mat format')
    out_pickled_extra_measures = File(
        'extra_measures',
        usedefault=True,
        desc=
        'Network measures for group 1 that return dictionaries stored as a Pickle.'
    )


class NetworkXMetricsOutputSpec(TraitedSpec):
    gpickled_network_files = OutputMultiPath(
        File(desc='Output gpickled network files'))
    matlab_matrix_files = OutputMultiPath(
        File(desc='Output network metrics in MATLAB .mat format'))
    global_measures_matlab = File(
        desc='Output global metrics in MATLAB .mat format')
    node_measures_matlab = File(
        desc='Output node metrics in MATLAB .mat format')
    edge_measures_matlab = File(
        desc='Output edge metrics in MATLAB .mat format')
    node_measure_networks = OutputMultiPath(
        File(desc='Output gpickled network files for all node-based measures'))
    edge_measure_networks = OutputMultiPath(
        File(desc='Output gpickled network files for all edge-based measures'))
    k_networks = OutputMultiPath(
        File(
            desc=
            'Output gpickled network files for the k-core, k-shell, and k-crust networks'
        ))
    k_core = File(desc='Computed k-core network stored as a NetworkX pickle.')
    k_shell = File(
        desc='Computed k-shell network stored as a NetworkX pickle.')
    k_crust = File(
        desc='Computed k-crust network stored as a NetworkX pickle.')
    pickled_extra_measures = File(
        desc=
        'Network measures for the group that return dictionaries, stored as a Pickle.'
    )
    matlab_dict_measures = OutputMultiPath(
        File(
            desc=
            'Network measures for the group that return dictionaries, stored as matlab matrices.'
        ))


class NetworkXMetrics(BaseInterface):
    """
    Calculates and outputs NetworkX-based measures for an input network

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> nxmetrics = cmtk.NetworkXMetrics()
    >>> nxmetrics.inputs.in_file = 'subj1.pck'
    >>> nxmetrics.run()                 # doctest: +SKIP
    """
    input_spec = NetworkXMetricsInputSpec
    output_spec = NetworkXMetricsOutputSpec

    def _run_interface(self, runtime):
        import scipy.io as sio
        global gpickled, nodentwks, edgentwks, kntwks, matlab
        gpickled = list()
        nodentwks = list()
        edgentwks = list()
        kntwks = list()
        matlab = list()
        ntwk = nx.read_gpickle(self.inputs.in_file)

        # Each block computes, writes, and saves a measure
        # The names are then added to the output .pck file list
        # In the case of the degeneracy networks, they are given specified output names

        calculate_cliques = self.inputs.compute_clique_related_measures
        weighted = self.inputs.treat_as_weighted_graph

        global_measures = compute_singlevalued_measures(
            ntwk, weighted, calculate_cliques)
        if isdefined(self.inputs.out_global_metrics_matlab):
            global_out_file = op.abspath(self.inputs.out_global_metrics_matlab)
        else:
            global_out_file = op.abspath(
                self._gen_outfilename('globalmetrics', 'mat'))
        sio.savemat(global_out_file, global_measures, oned_as='column')
        matlab.append(global_out_file)

        node_measures = compute_node_measures(ntwk, calculate_cliques)
        for key in list(node_measures.keys()):
            newntwk = add_node_data(node_measures[key], ntwk)
            out_file = op.abspath(self._gen_outfilename(key, 'pck'))
            nx.write_gpickle(newntwk, out_file)
            nodentwks.append(out_file)
        if isdefined(self.inputs.out_node_metrics_matlab):
            node_out_file = op.abspath(self.inputs.out_node_metrics_matlab)
        else:
            node_out_file = op.abspath(
                self._gen_outfilename('nodemetrics', 'mat'))
        sio.savemat(node_out_file, node_measures, oned_as='column')
        matlab.append(node_out_file)
        gpickled.extend(nodentwks)

        edge_measures = compute_edge_measures(ntwk)
        for key in list(edge_measures.keys()):
            newntwk = add_edge_data(edge_measures[key], ntwk)
            out_file = op.abspath(self._gen_outfilename(key, 'pck'))
            nx.write_gpickle(newntwk, out_file)
            edgentwks.append(out_file)
        if isdefined(self.inputs.out_edge_metrics_matlab):
            edge_out_file = op.abspath(self.inputs.out_edge_metrics_matlab)
        else:
            edge_out_file = op.abspath(
                self._gen_outfilename('edgemetrics', 'mat'))
        sio.savemat(edge_out_file, edge_measures, oned_as='column')
        matlab.append(edge_out_file)
        gpickled.extend(edgentwks)

        ntwk_measures = compute_network_measures(ntwk)
        for key in list(ntwk_measures.keys()):
            if key == 'k_core':
                out_file = op.abspath(
                    self._gen_outfilename(self.inputs.out_k_core, 'pck'))
            if key == 'k_shell':
                out_file = op.abspath(
                    self._gen_outfilename(self.inputs.out_k_shell, 'pck'))
            if key == 'k_crust':
                out_file = op.abspath(
                    self._gen_outfilename(self.inputs.out_k_crust, 'pck'))
            nx.write_gpickle(ntwk_measures[key], out_file)
            kntwks.append(out_file)
        gpickled.extend(kntwks)

        out_pickled_extra_measures = op.abspath(
            self._gen_outfilename(self.inputs.out_pickled_extra_measures,
                                  'pck'))
        dict_measures = compute_dict_measures(ntwk)
        iflogger.info('Saving extra measure file to %s in Pickle format',
                      op.abspath(out_pickled_extra_measures))
        with open(out_pickled_extra_measures, 'w') as fo:
            pickle.dump(dict_measures, fo)

        iflogger.info('Saving MATLAB measures as %s', matlab)

        # Loops through the measures which return a dictionary,
        # converts the keys and values to a Numpy array,
        # stacks them together, and saves them in a MATLAB .mat file via Scipy
        global dicts
        dicts = list()
        for idx, key in enumerate(dict_measures.keys()):
            for idxd, keyd in enumerate(dict_measures[key].keys()):
                if idxd == 0:
                    nparraykeys = np.array(keyd)
                    nparrayvalues = np.array(dict_measures[key][keyd])
                else:
                    nparraykeys = np.append(nparraykeys, np.array(keyd))
                    values = np.array(dict_measures[key][keyd])
                    nparrayvalues = np.append(nparrayvalues, values)
            nparray = np.vstack((nparraykeys, nparrayvalues))
            out_file = op.abspath(self._gen_outfilename(key, 'mat'))
            npdict = {}
            npdict[key] = nparray
            sio.savemat(out_file, npdict, oned_as='column')
            dicts.append(out_file)
        return runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outputs["k_core"] = op.abspath(
            self._gen_outfilename(self.inputs.out_k_core, 'pck'))
        outputs["k_shell"] = op.abspath(
            self._gen_outfilename(self.inputs.out_k_shell, 'pck'))
        outputs["k_crust"] = op.abspath(
            self._gen_outfilename(self.inputs.out_k_crust, 'pck'))
        outputs["gpickled_network_files"] = gpickled
        outputs["k_networks"] = kntwks
        outputs["node_measure_networks"] = nodentwks
        outputs["edge_measure_networks"] = edgentwks
        outputs["matlab_dict_measures"] = dicts
        outputs["global_measures_matlab"] = op.abspath(
            self._gen_outfilename('globalmetrics', 'mat'))
        outputs["node_measures_matlab"] = op.abspath(
            self._gen_outfilename('nodemetrics', 'mat'))
        outputs["edge_measures_matlab"] = op.abspath(
            self._gen_outfilename('edgemetrics', 'mat'))
        outputs["matlab_matrix_files"] = [
            outputs["global_measures_matlab"], outputs["node_measures_matlab"],
            outputs["edge_measures_matlab"]
        ]
        outputs["pickled_extra_measures"] = op.abspath(
            self._gen_outfilename(self.inputs.out_pickled_extra_measures,
                                  'pck'))
        return outputs

    def _gen_outfilename(self, name, ext):
        return name + '.' + ext


class AverageNetworksInputSpec(BaseInterfaceInputSpec):
    in_files = InputMultiPath(
        File(exists=True),
        mandatory=True,
        desc='Networks for a group of subjects')
    resolution_network_file = File(
        exists=True,
        desc=
        'Parcellation files from Connectome Mapping Toolkit. This is not necessary'
        ', but if included, the interface will output the statistical maps as networkx graphs.'
    )
    group_id = traits.Str('group1', usedefault=True, desc='ID for group')
    out_gpickled_groupavg = File(
        desc='Average network saved as a NetworkX .pck')
    out_gexf_groupavg = File(desc='Average network saved as a .gexf file')


class AverageNetworksOutputSpec(TraitedSpec):
    gpickled_groupavg = File(desc='Average network saved as a NetworkX .pck')
    gexf_groupavg = File(desc='Average network saved as a .gexf file')
    matlab_groupavgs = OutputMultiPath(
        File(desc='Average network saved as a .gexf file'))


class AverageNetworks(BaseInterface):
    """
    Calculates and outputs the average network given a set of input NetworkX gpickle files

    This interface will only keep an edge in the averaged network if that edge is present in
    at least half of the input networks.

    Example
    -------

    >>> import nipype.interfaces.cmtk as cmtk
    >>> avg = cmtk.AverageNetworks()
    >>> avg.inputs.in_files = ['subj1.pck', 'subj2.pck']
    >>> avg.run()                 # doctest: +SKIP

    """
    input_spec = AverageNetworksInputSpec
    output_spec = AverageNetworksOutputSpec

    def _run_interface(self, runtime):
        if isdefined(self.inputs.resolution_network_file):
            ntwk_res_file = self.inputs.resolution_network_file
        else:
            ntwk_res_file = self.inputs.in_files[0]

        global matlab_network_list
        network_name, matlab_network_list = average_networks(
            self.inputs.in_files, ntwk_res_file, self.inputs.group_id)
        return runtime

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if not isdefined(self.inputs.out_gpickled_groupavg):
            outputs["gpickled_groupavg"] = op.abspath(
                self._gen_outfilename(self.inputs.group_id + '_average',
                                      'pck'))
        else:
            outputs["gpickled_groupavg"] = op.abspath(
                self.inputs.out_gpickled_groupavg)

        if not isdefined(self.inputs.out_gexf_groupavg):
            outputs["gexf_groupavg"] = op.abspath(
                self._gen_outfilename(self.inputs.group_id + '_average',
                                      'gexf'))
        else:
            outputs["gexf_groupavg"] = op.abspath(
                self.inputs.out_gexf_groupavg)

        outputs["matlab_groupavgs"] = matlab_network_list
        return outputs

    def _gen_outfilename(self, name, ext):
        return name + '.' + ext



# -*- coding: utf-8 -*-
import os
from shutil import rmtree
from tempfile import mkdtemp

import nipype.interfaces.base as nib
import pytest
import nipype.pipeline.engine as pe


class InputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(desc='a random int')
    input2 = nib.traits.Int(desc='a random int')


class OutputSpec(nib.TraitedSpec):
    output1 = nib.traits.List(nib.traits.Int, desc='outputs')


class OarTestInterface(nib.BaseInterface):
    input_spec = InputSpec
    output_spec = OutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output1'] = [1, self.inputs.input1]
        return outputs


@pytest.mark.xfail(reason="not known")
def test_run_oar():
    cur_dir = os.getcwd()
    temp_dir = mkdtemp(prefix='test_engine_', dir=os.getcwd())
    os.chdir(temp_dir)

    pipe = pe.Workflow(name='pipe')
    mod1 = pe.Node(interface=OarTestInterface(), name='mod1')
    mod2 = pe.MapNode(
        interface=OarTestInterface(), iterfield=['input1'], name='mod2')
    pipe.connect([(mod1, mod2, [('output1', 'input1')])])
    pipe.base_dir = os.getcwd()
    mod1.inputs.input1 = 1
    execgraph = pipe.run(plugin="OAR")
    names = [
        '.'.join((node._hierarchy, node.name)) for node in execgraph.nodes()
    ]
    node = list(execgraph.nodes())[names.index('pipe.mod1')]
    result = node.get_output('output1')
    assert result == [1, 1]
    os.chdir(cur_dir)
    rmtree(temp_dir)



# -*- coding: utf-8 -*-
"""
Interfaces to the reconstruction algorithms in dipy

"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from future import standard_library
standard_library.install_aliases()
from builtins import str, open

import os.path as op

import numpy as np
import nibabel as nb
from distutils.version import LooseVersion

from ... import logging
from ..base import TraitedSpec, File, traits, isdefined
from .base import (DipyDiffusionInterface, DipyBaseInterfaceInputSpec,
                   HAVE_DIPY, dipy_version, dipy_to_nipype_interface,
                   get_dipy_workflows)


IFLOGGER = logging.getLogger('nipype.interface')

if HAVE_DIPY and LooseVersion(dipy_version()) >= LooseVersion('0.15'):
    from dipy.workflows import reconst

    l_wkflw = get_dipy_workflows(reconst)
    for name, obj in l_wkflw:
        new_name = name.replace('Flow', '')
        globals()[new_name] = dipy_to_nipype_interface(new_name, obj)
    del l_wkflw

else:
    IFLOGGER.info("We advise you to upgrade DIPY version. This upgrade will"
                  " open access to more models")


class RESTOREInputSpec(DipyBaseInterfaceInputSpec):
    in_mask = File(exists=True, desc=('input mask in which compute tensors'))
    noise_mask = File(
        exists=True, desc=('input mask in which compute noise variance'))


class RESTOREOutputSpec(TraitedSpec):
    fa = File(desc='output fractional anisotropy (FA) map computed from '
              'the fitted DTI')
    md = File(desc='output mean diffusivity (MD) map computed from the '
              'fitted DTI')
    rd = File(desc='output radial diffusivity (RD) map computed from '
              'the fitted DTI')
    mode = File(desc=('output mode (MO) map computed from the fitted DTI'))
    trace = File(
        desc=('output the tensor trace map computed from the '
              'fitted DTI'))
    evals = File(desc=('output the eigenvalues of the fitted DTI'))
    evecs = File(desc=('output the eigenvectors of the fitted DTI'))


class RESTORE(DipyDiffusionInterface):
    """
    Uses RESTORE [Chang2005]_ to perform DTI fitting with outlier detection.
    The interface uses :py:mod:`dipy`, as explained in `dipy's documentation`_.

    .. [Chang2005] Chang, LC, Jones, DK and Pierpaoli, C. RESTORE: robust \
    estimation of tensors by outlier rejection. MRM, 53:1088-95, (2005).

    .. _dipy's documentation: \
    http://nipy.org/dipy/examples_built/restore_dti.html


    Example
    -------

    >>> from nipype.interfaces import dipy as ndp
    >>> dti = ndp.RESTORE()
    >>> dti.inputs.in_file = '4d_dwi.nii'
    >>> dti.inputs.in_bval = 'bvals'
    >>> dti.inputs.in_bvec = 'bvecs'
    >>> res = dti.run() # doctest: +SKIP


    """
    input_spec = RESTOREInputSpec
    output_spec = RESTOREOutputSpec

    def _run_interface(self, runtime):
        from scipy.special import gamma
        from dipy.reconst.dti import TensorModel
        import gc

        img = nb.load(self.inputs.in_file)
        hdr = img.header.copy()
        affine = img.affine
        data = img.get_data()
        gtab = self._get_gradient_table()

        if isdefined(self.inputs.in_mask):
            msk = nb.load(self.inputs.in_mask).get_data().astype(np.uint8)
        else:
            msk = np.ones(data.shape[:3], dtype=np.uint8)

        try_b0 = True
        if isdefined(self.inputs.noise_mask):
            noise_msk = nb.load(self.inputs.noise_mask).get_data().reshape(-1)
            noise_msk[noise_msk > 0.5] = 1
            noise_msk[noise_msk < 1.0] = 0
            noise_msk = noise_msk.astype(np.uint8)
            try_b0 = False
        elif np.all(data[msk == 0, 0] == 0):
            IFLOGGER.info('Input data are masked.')
            noise_msk = msk.reshape(-1).astype(np.uint8)
        else:
            noise_msk = (1 - msk).reshape(-1).astype(np.uint8)

        nb0 = np.sum(gtab.b0s_mask)
        dsample = data.reshape(-1, data.shape[-1])

        if try_b0 and (nb0 > 1):
            noise_data = dsample.take(
                np.where(gtab.b0s_mask), axis=-1)[noise_msk == 0, ...]
            n = nb0
        else:
            nodiff = np.where(~gtab.b0s_mask)
            nodiffidx = nodiff[0].tolist()
            n = 20 if len(nodiffidx) >= 20 else len(nodiffidx)
            idxs = np.random.choice(nodiffidx, size=n, replace=False)
            noise_data = dsample.take(idxs, axis=-1)[noise_msk == 1, ...]

        # Estimate sigma required by RESTORE
        mean_std = np.median(noise_data.std(-1))
        try:
            bias = (1. - np.sqrt(2. / (n - 1)) * (gamma(n / 2.) / gamma(
                (n - 1) / 2.)))
        except:
            bias = .0
            pass

        sigma = mean_std * (1 + bias)

        if sigma == 0:
            IFLOGGER.warning('Noise std is 0.0, looks like data was masked and '
                             'noise cannot be estimated correctly. Using default '
                             'tensor model instead of RESTORE.')
            dti = TensorModel(gtab)
        else:
            IFLOGGER.info('Performing RESTORE with noise std=%.4f.', sigma)
            dti = TensorModel(gtab, fit_method='RESTORE', sigma=sigma)

        try:
            fit_restore = dti.fit(data, msk)
        except TypeError:
            dti = TensorModel(gtab)
            fit_restore = dti.fit(data, msk)

        hdr.set_data_dtype(np.float32)
        hdr['data_type'] = 16

        for k in self._outputs().get():
            scalar = getattr(fit_restore, k)
            hdr.set_data_shape(np.shape(scalar))
            nb.Nifti1Image(scalar.astype(np.float32), affine, hdr).to_filename(
                self._gen_filename(k))

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        for k in list(outputs.keys()):
            outputs[k] = self._gen_filename(k)
        return outputs


class EstimateResponseSHInputSpec(DipyBaseInterfaceInputSpec):
    in_evals = File(
        exists=True, mandatory=True, desc=('input eigenvalues file'))
    in_mask = File(
        exists=True, desc=('input mask in which we find single fibers'))
    fa_thresh = traits.Float(0.7, usedefault=True, desc=('FA threshold'))
    roi_radius = traits.Int(
        10, usedefault=True, desc=('ROI radius to be used in auto_response'))
    auto = traits.Bool(
        xor=['recursive'], desc='use the auto_response estimator from dipy')
    recursive = traits.Bool(
        xor=['auto'], desc='use the recursive response estimator from dipy')
    response = File(
        'response.txt', usedefault=True, desc=('the output response file'))
    out_mask = File('wm_mask.nii.gz', usedefault=True, desc='computed wm mask')


class EstimateResponseSHOutputSpec(TraitedSpec):
    response = File(exists=True, desc=('the response file'))
    out_mask = File(exists=True, desc=('output wm mask'))


class EstimateResponseSH(DipyDiffusionInterface):
    """
    Uses dipy to compute the single fiber response to be used in spherical
    deconvolution methods, in a similar way to MRTrix's command
    ``estimate_response``.


    Example
    -------

    >>> from nipype.interfaces import dipy as ndp
    >>> dti = ndp.EstimateResponseSH()
    >>> dti.inputs.in_file = '4d_dwi.nii'
    >>> dti.inputs.in_bval = 'bvals'
    >>> dti.inputs.in_bvec = 'bvecs'
    >>> dti.inputs.in_evals = 'dwi_evals.nii'
    >>> res = dti.run() # doctest: +SKIP


    """
    input_spec = EstimateResponseSHInputSpec
    output_spec = EstimateResponseSHOutputSpec

    def _run_interface(self, runtime):
        from dipy.core.gradients import GradientTable
        from dipy.reconst.dti import fractional_anisotropy, mean_diffusivity
        from dipy.reconst.csdeconv import recursive_response, auto_response

        img = nb.load(self.inputs.in_file)
        imref = nb.four_to_three(img)[0]
        affine = img.affine

        if isdefined(self.inputs.in_mask):
            msk = nb.load(self.inputs.in_mask).get_data()
            msk[msk > 0] = 1
            msk[msk < 0] = 0
        else:
            msk = np.ones(imref.shape)

        data = img.get_data().astype(np.float32)
        gtab = self._get_gradient_table()

        evals = np.nan_to_num(nb.load(self.inputs.in_evals).get_data())
        FA = np.nan_to_num(fractional_anisotropy(evals)) * msk
        indices = np.where(FA > self.inputs.fa_thresh)
        S0s = data[indices][:, np.nonzero(gtab.b0s_mask)[0]]
        S0 = np.mean(S0s)

        if self.inputs.auto:
            response, ratio = auto_response(
                gtab,
                data,
                roi_radius=self.inputs.roi_radius,
                fa_thr=self.inputs.fa_thresh)
            response = response[0].tolist() + [S0]
        elif self.inputs.recursive:
            MD = np.nan_to_num(mean_diffusivity(evals)) * msk
            indices = np.logical_or(FA >= 0.4,
                                    (np.logical_and(FA >= 0.15, MD >= 0.0011)))
            data = nb.load(self.inputs.in_file).get_data()
            response = recursive_response(
                gtab,
                data,
                mask=indices,
                sh_order=8,
                peak_thr=0.01,
                init_fa=0.08,
                init_trace=0.0021,
                iter=8,
                convergence=0.001,
                parallel=True)
            ratio = abs(response[1] / response[0])
        else:
            lambdas = evals[indices]
            l01 = np.sort(np.mean(lambdas, axis=0))

            response = np.array([l01[-1], l01[-2], l01[-2], S0])
            ratio = abs(response[1] / response[0])

        if ratio > 0.25:
            IFLOGGER.warning('Estimated response is not prolate enough. '
                             'Ratio=%0.3f.', ratio)
        elif ratio < 1.e-5 or np.any(np.isnan(response)):
            response = np.array([1.8e-3, 3.6e-4, 3.6e-4, S0])
            IFLOGGER.warning(
                'Estimated response is not valid, using a default one')
        else:
            IFLOGGER.info('Estimated response: %s', str(response[:3]))

        np.savetxt(op.abspath(self.inputs.response), response)

        wm_mask = np.zeros_like(FA)
        wm_mask[indices] = 1
        nb.Nifti1Image(wm_mask.astype(np.uint8), affine, None).to_filename(
            op.abspath(self.inputs.out_mask))
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['response'] = op.abspath(self.inputs.response)
        outputs['out_mask'] = op.abspath(self.inputs.out_mask)
        return outputs


class CSDInputSpec(DipyBaseInterfaceInputSpec):
    in_mask = File(exists=True, desc=('input mask in which compute tensors'))
    response = File(exists=True, desc=('single fiber estimated response'))
    sh_order = traits.Int(
        8, usedefault=True, desc=('maximal shperical harmonics order'))
    save_fods = traits.Bool(True, usedefault=True, desc=('save fODFs in file'))
    out_fods = File(desc=('fODFs output file name'))


class CSDOutputSpec(TraitedSpec):
    model = File(desc='Python pickled object of the CSD model fitted.')
    out_fods = File(desc=('fODFs output file name'))


class CSD(DipyDiffusionInterface):
    """
    Uses CSD [Tournier2007]_ to generate the fODF of DWIs. The interface uses
    :py:mod:`dipy`, as explained in `dipy's CSD example
    <http://nipy.org/dipy/examples_built/reconst_csd.html>`_.

    .. [Tournier2007] Tournier, J.D., et al. NeuroImage 2007.
      Robust determination of the fibre orientation distribution in diffusion
      MRI: Non-negativity constrained super-resolved spherical deconvolution


    Example
    -------

    >>> from nipype.interfaces import dipy as ndp
    >>> csd = ndp.CSD()
    >>> csd.inputs.in_file = '4d_dwi.nii'
    >>> csd.inputs.in_bval = 'bvals'
    >>> csd.inputs.in_bvec = 'bvecs'
    >>> res = csd.run() # doctest: +SKIP
    """
    input_spec = CSDInputSpec
    output_spec = CSDOutputSpec

    def _run_interface(self, runtime):
        from dipy.reconst.csdeconv import ConstrainedSphericalDeconvModel
        from dipy.data import get_sphere
        # import marshal as pickle
        import pickle as pickle
        import gzip

        img = nb.load(self.inputs.in_file)
        imref = nb.four_to_three(img)[0]

        if isdefined(self.inputs.in_mask):
            msk = nb.load(self.inputs.in_mask).get_data()
        else:
            msk = np.ones(imref.shape)

        data = img.get_data().astype(np.float32)

        gtab = self._get_gradient_table()
        resp_file = np.loadtxt(self.inputs.response)

        response = (np.array(resp_file[0:3]), resp_file[-1])
        ratio = response[0][1] / response[0][0]

        if abs(ratio - 0.2) > 0.1:
            IFLOGGER.warning('Estimated response is not prolate enough. '
                             'Ratio=%0.3f.', ratio)

        csd_model = ConstrainedSphericalDeconvModel(
            gtab, response, sh_order=self.inputs.sh_order)

        IFLOGGER.info('Fitting CSD model')
        csd_fit = csd_model.fit(data, msk)

        f = gzip.open(self._gen_filename('csdmodel', ext='.pklz'), 'wb')
        pickle.dump(csd_model, f, -1)
        f.close()

        if self.inputs.save_fods:
            sphere = get_sphere('symmetric724')
            fods = csd_fit.odf(sphere)
            nb.Nifti1Image(fods.astype(np.float32), img.affine,
                           None).to_filename(self._gen_filename('fods'))

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['model'] = self._gen_filename('csdmodel', ext='.pklz')
        if self.inputs.save_fods:
            outputs['out_fods'] = self._gen_filename('fods')
        return outputs
